# PMAlignACE Package
# Function: routines for evaluating AlignACE motif discovery algorithm
#
# Author: Jianjun Hu (hujianju@purdue.edu)
# Kihara Laboratory of Bioinformatics
# Department of Biological Sciences
# Purdue University, West Lafayette, IN, 47906.
#
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
# Last modified: 10-25-2004

package PMAlignACE;
use PMotif;
use strict;


#------------------------------------------------------------------------
#function: extract all the discovered motifs from output of 
#          AlignACE: each motif info seperated by ';' 
#          motif position for each sequence separated by ','
#          startPosition and width and score separated by ' '
#input:    filename of alignACE output file, number of motifs to report
#comment:  for AlignACE results, we skip the motifs found in reverse sequence
#return:   an array of motif information.
#          seqID startPos width, ...: score    for each motif discovered
#		   1  123232 13, 2 232323 13, ...:13.9
#		   2  23232 12, 3 234553 13, ...: 24.9

sub parseMotifInfo{
    my $nMotif; #number of motifs to be reported
    my $filename;
    if(scalar @_==1){
	$nMotif=0;
	($filename)=@_;
    }
    else{
        ($filename,$nMotif)=@_;
    }
    unless(open(MOTIFFILE,$filename)){
        print "PMAlignACE::parseMotifInfo: Cannot open file $filename!\n";
    }
    my $filedata=join('',<MOTIFFILE>);

    $filedata=~/.*?(Motif.*)/ms;
    my $motifs=$1;
    #store motif information to be returned
    my @motifInfo=();
    
    my $count=1;
    while($motifs=~/(^Motif.*?)MAP Score: (\d*\.\d*)/sgm){
		if($count>$nMotif && $nMotif!=0){
			last;
		}
		else{
			$count+=1;
		}
		
		my $motif=$1;
		my $score=$2;
		my @motiflines=split(/\n/,$motif);
		shift @motiflines;#remove Motif i
		pop   @motiflines;#remove ******
	
		my @exportRecords=();
		#build seqName to seqID mapping hash
		$filename=~/.*\/(.*)/; #./step5regulon/AraC.txt
		#strip the filename, for ensemble algorithm we have 0-filename
		my $srcFilename=$1;
		my $seqFile="./step5regulon/$srcFilename";	#default
		
		if($srcFilename =~/(\D*)-.*/|| $srcFilename =~/\d*-(.*)/g){
			#ArcA-0.txt for ensemble algorithm
			$seqFile = "./step5regulon/$1";
		}
		
		unless (open(TRAIN_SEQ_FILE,"$seqFile")){
			print "Error in PMAlignACE: cannot open file $seqFile\" !\n";
			exit;
		}
		my @trainSeqs=<TRAIN_SEQ_FILE>;
		close(TRAIN_SEQ_FILE);
		
		my %seqIDfromName=();
		my $id=0;
		foreach (@trainSeqs){
			#use the starting position of the motif as index keyword
			my $seqName=$_;
			if($_=~/>.*/sm){
				my @items=split(" ",$seqName);
				substr($items[0],0,1)="";
				$seqName=$items[0];
				$seqIDfromName{$seqName}=$id;
				$id++;
			}
		}	
	
		foreach my $motifline (@motiflines){
			my @features=split(' ',$motifline);
			#only count the forward direction motifs
			if($features[3]==1){
				my $width=length($features[0]);
				my $seqID =$features[1]; #for alignACE seqID = $features[1]
				push(@exportRecords,"$features[1] $features[2] $width");
			}
		}
		
		my $itemlist= join(',',@exportRecords);
		if(scalar @exportRecords!=0){
			push(@motifInfo, "$itemlist:$score");
		}
    }
    return @motifInfo;
}


sub getAccuracy{
	return PMotif::getAccuracy(@_,"AlignACE");
}


#get the average accuracy of the prediction for a set of sequences
sub getAccuracyAve{
    return PMotif::getAccuracyAve(@_,"AlignACE" );    
}


#------------------------------------------------------------------------
#function: calculate the convergence and accuracy for all motifs in an output file of AlignACE
#input:    a filename of alignACE output
#          $nScore Number of scores to report
#          $nMotif number of motifs to search
#          $voteThreshold percentage of voting sequences to be regarded as consensus motif region
#return:   accuracy values and convergence score  for the discovered motifs along with its statistical scores
#          $aveAccuracy $score $nSeq $motifWidth
sub getConvergeAccuracy{
	return PMotif::getConvergeAccuracy(@_,"AlignACE");
}

sub getConvergeAveAccuracy{
	return PMotif::getConvergeAveAccuracy(@_,"AlignACE");
}

1;

__END__

=head1 NAME

PMAlignACE subroutines
