# PMBioProspector Package
# Function: routines for evaluating BioProspector motif discovery algorithms
#
# Author: Jianjun Hu (hujianju@purdue.edu)
# Kihara Laboratory of Bioinformatics
# Department of Biological Sciences
# Purdue University, West Lafayette, IN, 47906.
#
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
# Last modified: 10-25-2004

package PMBioProspector;
use PMotif;
use strict;


#------------------------------------------------------------------------
#function: extract all the discovered motifs from output of 
#          BioProspector: each motif info seperated by ';'
#          motif position for each sequence separated by ','
#          startPosition and width and score separated by ' '
#input:    filename of BioProspector output file, number of motifs to report
#comment:  for BioProspector results, we skip the motifs found in reverse sequence
#return:   an array of motif information.
#          seqID startPos width, ...: score    for each motif discovered

sub parseMotifInfo{
    my $nMotif; #number of motifs to be reported
    my $filename;
    if(scalar @_==1){
	$nMotif=0;
	($filename)=@_;
    }
    else{
        ($filename,$nMotif)=@_;
    }
    unless(open(MOTIFFILE,$filename)){
        print "PMotif::parseMotifInfo: Cannot open file $filename!\n";
	exit;
    }
    my $filedata=join('',<MOTIFFILE>);
    my @motifs=split("Motif #",$filedata);
    shift(@motifs);
    

    #store motif information to be returned
    my @motifInfo=();
    my $count=1;
    foreach (@motifs){
	if($count>$nMotif && $nMotif!=0){
	    last;
	}
	else{
	    $count+=1;
	}
        my $motif=$_;
	$motif=~/.*Width \((\d*),.*MotifScore (.*);(.*)/mgs;
	
	my $motifWidth=$1;
	my $score=$2;
	my $motifRest=$3;
	
    my @motiflines=split(/\n/,$motifRest);
	@motiflines = grep{$_=~/^>.*/}@motiflines;
	
	#build the hash for finding the sequence ID number by its name
	#Load the training sequence file and build a hash
	#2308475-->0
        #load the experimental motif data

	my @exportRecords=();
	$filename=~/.*\/(.*)/;
	#strip the filename, for ensemble algorithm we have 0-filename
	my $srcFilename=$1;
	$srcFilename =~/\d*-(.*)/g;
	my $seqFile="./step5regulon/$1";
	if($srcFilename =~/(\D*)-.*/){#ArcA-0.txt
		$seqFile = "./step5regulon/$srcFilename";
	}

	unless (open(TRAIN_SEQ_FILE,"$seqFile")){
		print "PMBioProspector: cannot open file \"$filename\" !\n";
		exit;
	}
	
	my @trainSeqs=<TRAIN_SEQ_FILE>;
	close(TRAIN_SEQ_FILE);
	my %seqIDfromName=();
	my $id=0;
	foreach (@trainSeqs){
	    #use the starting position of the motif as index keyword
	    my $seqName=$_;
	    if($_=~/>.*/sm){
		my @items=split(" ",$seqName);
		substr($items[0],0,1)="";
		$seqName=$items[0];
		$seqIDfromName{$seqName}=$id;
		$id++;
	    }
	}

	foreach my $motifline (@motiflines){
		my @features=split('\t',$motifline);
		my @posItems= split(" ",$features[3]);
		my $startPos= $posItems[1];
		#we don't count the motifs found in complementary reverse
		#sequences
		if($posItems[0] eq "r"){
		next;
		}

		my @items=split(" ",$features[0]);
		my $seqName=$items[0];
		substr($seqName,0,1)="";#remove >
		my $seqID=$seqIDfromName{$seqName};
		push(@exportRecords,"$seqID $startPos $motifWidth");
	}
	
	my $itemlist= join(',',@exportRecords);
	if(scalar @exportRecords!=0){
	    push(@motifInfo, "$itemlist:$score");
	}
    }
    return @motifInfo;
}



#parsing function for version 1.0
sub parseMotifInfo_v1{
    my $nMotif; #number of motifs to be reported
    my $filename;
    if(scalar @_==1){
	$nMotif=0;
	($filename)=@_;
    }
    else{
        ($filename,$nMotif)=@_;
    }
    unless(open(MOTIFFILE,$filename)){
        print "PMotif::parseMotifInfo: Cannot open file $filename!\n";
	exit;
    }
    my $filedata=join('',<MOTIFFILE>);
    my @motifs=split("Motif #",$filedata);
    shift(@motifs);
    

    #store motif information to be returned
    my @motifInfo=();
    my $count=1;
    foreach (@motifs){
	if($count>$nMotif && $nMotif!=0){
	    last;
	}
	else{
	    $count+=1;
	}
        my $motif=$_;
	$motif=~/.*Width \((\d*),.*MotifScore (.*);(.*)/mgs;
	
	my $motifWidth=$1;
	my $score=$2;
	my $motifRest=$3;
	
        my @motiflines=split(/\n/,$motifRest);
	@motiflines = grep{$_=~/^>.*/}@motiflines;
	
	#build the hash for finding the sequence ID number by its name
	#Load the training sequence file and build a hash
	#2308475-->0
        #load the experimental motif data

	my @exportRecords=();
	$filename=~/\.\/.*\/(.*)/;
	#strip the filename, for ensemble algorithm we have 0-filename
	my $srcFilename=$1;
	$srcFilename =~/\d*-(.*)/g;
	
	my $seqFile="./step5regulon/$1";
	unless (open(TRAIN_SEQ_FILE,"$seqFile")){
		print "PMBioProspector: cannot open file \"$filename\" !\n";
		exit;
	}
	
	my @trainSeqs=<TRAIN_SEQ_FILE>;
	close(TRAIN_SEQ_FILE);
	my %seqIDfromName=();
	my $id=0;
	foreach (@trainSeqs){
	    #use the starting position of the motif as index keyword
	    my $seqName=$_;
	    if($_=~/>.*/sm){
		my @items=split(" ",$seqName);
		substr($items[0],0,1)="";
		$seqName=$items[0];
		$seqIDfromName{$seqName}=$id;
		$id++;
	    }
	}

        foreach my $motifline (@motiflines){
            my @features=split('\t',$motifline);
	    $features[2]=~/([f|r])(.*)/;
	    my $startPos= $2;
	    
	    #we don't count the motifs found in complementary reverse
	    #sequences
	    if($1 eq "r"){
		next;
	    }

	    my @items=split(" ",$features[0]);
	    my $seqName=$items[0];
	    substr($seqName,0,1)="";#remove >
	    my $seqID=$seqIDfromName{$seqName};
	    push(@exportRecords,"$seqID $startPos $motifWidth");
        }
	
	my $itemlist= join(',',@exportRecords);
	if(scalar @exportRecords!=0){
	    push(@motifInfo, "$itemlist:$score");
	}
    }
    return @motifInfo;
}
#------------------------------------------------------------------------
#function: calculate the accuracy for all motifs in an output file of BioProspector
#input:    a filename of BioProspector output
#          nMotifs: number of motifs to be searched
#          nScore: number of scores to be reported for the discovered motifs
#return:   accuracy values for the discovered motifs along with its statistical scores
#          $aveAccuracy $score $nSeq $motifWidth
#------------------------------------------------------------------------
#function: calculate the accuracy for all motifs in an output file of MotifSampler
#input:    a filename of MotifSampler output
#          nMotifs: number of motifs to be searched
#          nScore: number of scores to be reported for the discovered motifs
#return:   accuracy values for the discovered motifs along with its statistical scores
#          $aveAccuracy $score $nSeq $motifWidth
sub getAccuracy{
    return PMotif::getAccuracy(@_,"BioProspector" );    
}


#get the average accuracy of the prediction for a set of sequences
sub getAccuracyAve{
    return PMotif::getAccuracyAve(@_,"BioProspector" );    
}

#------------------------------------------------------------------------
#function: calculate the convergence and accuracy for all motifs in an output file of AlignACE
#input:    a filename of BioProspector output
#          $nScore Number of scores to report
#          $nMotif number of motifs to search
#          $voteThreshold percentage of voting sequences to be regarded as consensus motif region
#return:   accuracy values and convergence score  for the discovered motifs along with its statistical scores
#          $aveAccuracy $score $nSeq $motifWidth
sub getConvergeAccuracy{
	return PMotif::getConvergeAccuracy(@_,"BioProspector");
}

#------------------------------------------------------------------------
#function: calculate the convergence and accuracy for all motifs in an output file of BioProspector
#input:    a filename of BioProspector output
#return:   accuracy values and convergence score  for the discovered motifs along with its statistical scores
#          $aveAccuracy $score $nSeq $motifWidth
sub getConvergeAveAccuracy{
	return PMotif::getConvergeAveAccuracy(@_,"BioProspector");    
}


1;

__END__

=head1 NAME

PMBioProspector subroutines
