# PMMDScan Package
# Function: routines for evaluating MDScan motif discovery algorithm
#
# Author: Jianjun Hu (hujianju@purdue.edu)
# Kihara Laboratory of Bioinformatics
# Department of Biological Sciences
# Purdue University, West Lafayette, IN, 47906.
#
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
# Last modified: 10-25-2004v


package PMMDScan;
use PMotif;
use strict;


#------------------------------------------------------------------------
#function: extract all the discovered motifs from output of 
#          MDScan: each motif info seperated by ';'
#          motif position for each sequence separated by ','
#          startPosition and width and score separated by ' '
#input:    filename of MDScan output file, number of motifs to report
#comment:  for MDScan results, we skip the motifs found in reverse sequence
#return:   an array of motif information.
#          seqID startPos width, ...: score    for each motif discovered

sub parseMotifInfo{
    my $nMotif; #number of motifs to be reported
    my $filename;
    if(scalar @_==1){
	$nMotif=0;
	($filename)=@_;
    }
    else{
        ($filename,$nMotif)=@_;
    }
	#print "filename $filename\n";
	
    unless(open(MOTIFFILE,$filename)){
        print "PMDScan::parseMotifInfo: Cannot open file $filename! -1\n";
		exit;
    }
    my $filedata=join('',<MOTIFFILE>);

    $filedata=~/^(Mtf.*)Total time/ms;


    my $motifs=$1;
    my $endTag="Mtf";
    $motifs="$motifs$endTag"; #add parsing tags at the end
    #store motif information to be returned
    my @motifInfo=();
   
    my $count=1;
    while($motifs=~/^(Mtf.*?)(?=Mtf)/sgm){ #look ahead technique
	if($count>$nMotif && $nMotif!=0){
	    last;
	}
	else{
	    $count+=1;
	}
        my $motif=$1;
    #my $score=$2;
	$motif=~/^Mtf(.*)\nFinal (.*)/s;
	my ($index,$motifWidth,$score,$segments)=split(/\t/,$1);
	my $motifRest=$2;

        $motifRest=~/Motif.*?(?=>)(.*)/s;
	my $motifSet=$1;
        my @motiflines=split(/\n/,$motifSet);

        my @exportRecords=();
	
	#build the hash for finding the sequence ID number by its name
	#Load the training sequence file and build a hash
	#2308475 2308502 reverse   -->0
        #load the experimental motif data

    $filename=~/.*\/(.*)/;
	
	#strip the filename, for ensemble algorithm we have 0-filename
	my $srcFilename=$1;
	$srcFilename =~/\d*-(.*)/g;	   #0-ArcA.txt for ensemble algorithm
    my $seqFile="./step5regulon/$1";
	if($srcFilename =~/(\D*)-.*/){#ArcA-0.txt
		$seqFile = "./step5regulon/$srcFilename";
	}
	
	unless (open(TRAIN_SEQ_FILE,"$seqFile")){
	    print "PMDScan::parseMotifInfo: cannot open file \"$seqFile\" ! -2\n";
	    exit;
	}
	my @trainSeqs=<TRAIN_SEQ_FILE>;
	close(TRAIN_SEQ_FILE);
	my %seqIDfromName=();
	my $id=0;
	foreach (@trainSeqs){
	    my $seqName=$_;
	    if($_=~/>.*/sm){
		chomp($seqName);
		$seqIDfromName{$seqName}=$id;
		$id++;
	    }
	}

        foreach my $motifline (@motiflines){
	    my $startPos;
            my @features=split('\t',$motifline);
	    my $seqName = $features[0];
	    #$seqName=~s/> //;
	    $features[1]=~/St ([fb])(\d*)/;
	    if($1 eq 'b') {
		next;
	    }
	    else{
		$startPos=$2;
		my $seqID=$seqIDfromName{$seqName};
                push(@exportRecords,"$seqID $startPos $motifWidth");
            }
        }
	
	my $itemlist= join(',',@exportRecords);
	if(scalar @exportRecords!=0){
	    push(@motifInfo, "$itemlist:$score");
	}
    }
    return @motifInfo;
}

#------------------------------------------------------------------------
#function: calculate the accuracy for all motifs in an output file of MDScan
#input:    a filename of MDScan output
#          nMotifs: number of motifs to be searched
#          nScore: number of scores to be reported for the discovered motifs
#return:   accuracy values for the discovered motifs along with its statistical scores
#          $aveAccuracy $score $nSeq $motifWidth
#------------------------------------------------------------------------
sub getAccuracy{
    return PMotif::getAccuracy(@_,"MDScan" );    
}

#get the average accuracy of the prediction for a set of sequences
sub getAccuracyAve{
    return PMotif::getAccuracyAve(@_,"MDScan" );    
}

#------------------------------------------------------------------------
#function: calculate the convergence and accuracy for all motifs in an output file of AlignACE
#input:    a filename of MDScan output
#          $nScore Number of scores to report
#          $nMotif number of motifs to search
#          $voteThreshold percentage of voting sequences to be regarded as consensus motif region
#return:   accuracy values and convergence score  for the discovered motifs along with its statistical scores
#          $aveAccuracy $score $nSeq $motifWidth
sub getConvergeAccuracy{
	return PMotif::getConvergeAccuracy(@_,"MDScan");
}

#------------------------------------------------------------------------
#function: calculate the convergence and accuracy for all motifs in an output file of BioProspector
#input:    a filename of MDScan output
#return:   accuracy values and convergence score  for the discovered motifs along with its statistical scores
#          $aveAccuracy $score $nSeq $motifWidth
sub getConvergeAveAccuracy{
	return PMotif::getConvergeAveAccuracy(@_,"MDScan");    
}

1;

__END__

=head1 NAME

PMMDScan subroutines
