# PMMeme Package
# Function: routines for evaluating MEME motif discovery algorithm
#
# Author: Jianjun Hu (hujianju@purdue.edu)
# Kihara Laboratory of Bioinformatics
# Department of Biological Sciences
# Purdue University, West Lafayette, IN, 47906.
#
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
# Last modified: 10-25-2004

package PMMeme;
use PMotif;
use strict;


#------------------------------------------------------------------------
#function: extract all the discovered motifs from output of 
#          Meme: each motif info seperated by ';' 
#          motif position for each sequence separated by ','
#          startPosition and width and score separated by ' '
#   input: filename of Meme output file, number of motifs to report
# comment: for Meme results, we skip the motifs found in reverse sequence
#  return: an array of motif information.
#          seqID startPos width, ...: score    for each motif discovered
#------------------------------------------------------------------------
sub parseMotifInfo{
    my $nMotif; #number of motifs to be reported
    my $filename;
    if(scalar @_==1){
        $nMotif=0;
        ($filename)=@_;
    }
    else{
        ($filename,$nMotif)=@_;
    }

    unless(open(MOTIFFILE,$filename)){
        print "PMMeme::parseMotifInfo: Cannot open file $filename!\n";
    }
    my $filedata=join('',<MOTIFFILE>);

    #print $filedata;
    $filedata=~/.*?(MOTIF.*)/sm;
    my $motifs=$1;
    #print $motifs;
    #store motif information to be returned
    my @motifInfo=();

    my $count=1;
    while ($motifs=~/(^MOTIF.*?secs.)/sgm){
        if($count>$nMotif && $nMotif!=0){
            last;
        }
        else{
            $count+=1;
        }
  
	my $motif=$1;
	$motif=~/(^MOTIF.*?\n).*?Site(.*).*block diagram.*/sm;
	my $part1=$1;
	my @record = split(/\n/,$2);

	$part1=~/^MOTIF(.*)width =(.*)sites =(.*)llr =(.*)E-value =(.*)\n/;
	my $width=$2;
	my $score=$5;
	#print $width,"***",,$2,$3,$5, $score,"\n";
	shift(@record);
	shift(@record);
	pop(@record); 
	pop(@record);
	pop(@record);
	pop(@record);

	#for motifs in each sequence, do the following
    my @exportRecords=();
    #build seqName to seqID mapping hash
    $filename=~/.*\/(.*)/; #./step5regulon/AraC.txt
    #strip the filename, for ensemble algorithm we have 0-filename
    my $srcFilename=$1;
    my $seqFile="./step5regulon/$srcFilename";	#default
    
    if($srcFilename =~/(\D*)-.*/|| $srcFilename =~/\d*-(.*)/g){
        #ArcA-0.txt for ensemble algorithm
        $seqFile = "./step5regulon/$1";
    }    
	
	unless (open(TRAIN_SEQ_FILE,"$seqFile")){
	    print "PMMeme::getAccuracy: cannot open file \"$filename\" !\n";
	    exit;
	}
	my @trainSeqs=<TRAIN_SEQ_FILE>;
	close(TRAIN_SEQ_FILE);
	my %seqIDfromName=();
	my $id=0;
	foreach (@trainSeqs){
	    #use the starting position of the motif as index keyword
	    my $seqName=$_;
	    if($_=~/>.*/sm){
		my @items=split(" ",$seqName);
		substr($items[0],0,1)="";
		$seqName=$items[0];
		$seqIDfromName{$seqName}=$id;
		$id++;
	    }
	}

	foreach my $line (@record){
	    my ($seqName, $startPos) = split(' ',$line);
	    my $seqID=$seqIDfromName{$seqName};
	    push(@exportRecords,"$seqID $startPos $width");
	}
	my $itemlist= join(',',@exportRecords);
	if(scalar @exportRecords!=0){
	    push(@motifInfo, "$itemlist:$score");
	}
    }
    return @motifInfo;
}

#------------------------------------------------------------------------
#function: calculate the accuracy for all motifs in an output file of AlignMEME
#input:    a filename of alignMEME output
#return:   accuracy values for the discovered motifs along with its statistical scores
#          $aveAccuracy $score $nSeq $motifWidth
sub getAccuracy{
    return PMotif::getAccuracy(@_,"MEME" );    
}

#get the average accuracy of the prediction for a set of sequences
sub getAccuracyAve{
    return PMotif::getAccuracyAve(@_,"MEME" );    
}
#------------------------------------------------------------------------
#function: calculate the convergence and accuracy for all motifs in an output file of AlignACE
#input:    a filename of alignACE output
#          $nScore Number of scores to report
#          $nMotif number of motifs to search
#          $voteThreshold percentage of voting sequences to be regarded as consensus motif region
#return:   accuracy values and convergence score  for the discovered motifs along with its statistical scores
#          $aveAccuracy $score $nSeq $motifWidth
#------------------------------------------------------------------------
sub getConvergeAccuracy{
	return PMotif::getConvergeAccuracy(@_,"MEME");
}

#function: calculate the convergence and accuracy for all motifs in an output file of AlignACE
#input:    a filename of alignACE output
#          $nScore Number of scores to report
#          $nMotif number of motifs to search
#          $voteThreshold percentage of voting sequences to be regarded as consensus motif region
#return:   accuracy values and convergence score  for the discovered motifs along with its statistical scores
#          $aveAccuracy $score $nSeq $motifWidth
sub getConvergeAveAccuracy{
	return PMotif::getConvergeAveAccuracy(@_,"MEME");    
}


1;

__END__

=head1 NAME

PMMeme subroutines
