# PMMotifSampler Package
# Function: routines for evaluating MotifSampler motif discovery algorithm
#
# Author: Jianjun Hu (hujianju@purdue.edu)
# Kihara Laboratory of Bioinformatics
# Department of Biological Sciences
# Purdue University, West Lafayette, IN, 47906.
#
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
# Last modified: 10-25-2004

package PMMotifSampler;
use PMotif;
use strict;


#------------------------------------------------------------------------
#function: extract all the discovered motifs from output of 
#          MotifSampler: each motif info seperated by ';'
#          motif position for each sequence separated by ','
#          startPosition and width and score separated by ' '
#input:    filename of MotifSampler output file, number of motifs to report
#comment:  for MotifSampler results, we skip the motifs found in reverse sequence
#return:   an array of motif information.
#          seqID startPos width, ...: score    for each motif discovered

sub parseMotifInfo{
    my $nMotif; #number of motifs to be reported
    my $filename;
    if(scalar @_==1){
	$nMotif=0;
	($filename)=@_;
    }
    else{
        ($filename,$nMotif)=@_;
    }
    unless(open(MOTIFFILE,$filename)){
        print "PMotifSampler::parseMotifInfo: Cannot open file $filename!\n";
	exit;
    }
    my $filedata=join('',<MOTIFFILE>);
    $filedata=~/^#INCLUSive GFF File\n(.*)/ms;
    my @motifs=split("#id: ",$1);
    shift(@motifs);

    #store motif information to be returned
    my @motifInfo=();
    my $count=1;
    foreach (@motifs){
	if($count>$nMotif && $nMotif!=0){
	    last;
	}
	else{
	    $count+=1;
	}
        my $motif=$_;
	#$motif=~/^.*consensus: (.*)\tsequences:.*cs: (.*)\tic:.*?\n(.*)/mgs;
	$motif=~/^.*consensus: (.*)\tsequences:.*?cs: (.*?)\tic: (.*?)\tll: (.*?)\n(.*)/mgs;
	#										  consensus score  information content  log likelyhoods
	
	my $motifWidth=length($1);
	my $score=$2;
	my $motifRest=$5;
    my @motiflines=split(/\n/,$motifRest);

    my @exportRecords=();
	
	#build the hash for finding the sequence ID number by its name
	#Load the training sequence file and build a hash
	#2308475-->0
	#load the experimental motif data

	$filename=~/.*\/(.*)/;
	#strip the filename, for ensemble algorithm we have 0-filename
	my $srcFilename=$1;
	$srcFilename =~/\d*-(.*)/g;
	my $seqFile="./step5regulon/$1";
	if($srcFilename =~/(\D*)-.*/){#ArcA-0.txt
		$seqFile = "./step5regulon/$srcFilename";
	}
	
	unless (open(TRAIN_SEQ_FILE,"$seqFile")){
	    print "Error in PMDMotifSampler: cannot open file \"$filename\" !\n";
	    exit;
	}
	my @trainSeqs=<TRAIN_SEQ_FILE>;
	close(TRAIN_SEQ_FILE);
	my %seqIDfromName=();
	my $id=0;
	foreach (@trainSeqs){
	    #use the starting position of the motif as index keyword
	    my $seqName=$_;
	    if($_=~/>.*/sm){
		my @items=split(" ",$seqName);
		substr($items[0],0,1)="";
		$seqName=$items[0];
		$seqIDfromName{$seqName}=$id;
		$id++;
	    }
	}

        foreach my $motifline (@motiflines){
            my @features=split('\t',$motifline);
	    my $seqName = $features[0];
	    my $startPos= $features[3];
	    my $endPos =  $features[4];
	    my $motifWidth = $endPos-$startPos+1;
	    my $seqID=$seqIDfromName{$seqName};
	    push(@exportRecords,"$seqID $startPos $motifWidth");
        }
	
	my $itemlist= join(',',@exportRecords);
	if(scalar @exportRecords!=0){
	    push(@motifInfo, "$itemlist:$score");
	}
    }
    return @motifInfo;
}

#------------------------------------------------------------------------
#function: calculate the accuracy predicting motifs in all sequences in an output file of MotifSampler
#input:    a filename of MotifSampler output
#          nMotifs: number of motifs to be searched
#          nScore: number of scores to be reported for the discovered motifs
#return:   accuracy values for the discovered motifs along with its statistical scores
#          $aveAccuracy $score $nSeq $motifWidth
sub getAccuracy{
    return PMotif::getAccuracy(@_,"MotifSampler" );    
}


#get the average accuracy of the prediction for a set of sequences
sub getAccuracyAve{
    return PMotif::getAccuracyAve(@_,"MotifSampler" );    
}

#------------------------------------------------------------------------
#function: calculate the convergence and accuracy for all motifs in an output file of MotifSampler
#input:    a filename of MotifSampler output
#return:   accuracy values and convergence score  for the discovered motifs along with its statistical scores
#          $aveAccuracy $score $nSeq $motifWidth
sub getConvergeAccuracy{
	return PMotif::getConvergeAccuracy(@_,"MotifSampler");    
}

#------------------------------------------------------------------------
#function: calculate the convergence and accuracy for all motifs in an output file of BioProspector
#input:    a filename of BioProspector output
#return:   accuracy values and convergence score  for the discovered motifs along with its statistical scores
#          $aveAccuracy $score $nSeq $motifWidth
sub getConvergeAveAccuracy{
	return PMotif::getConvergeAveAccuracy(@_,"MotifSampler");    
}


1;

__END__

=head1 NAME

PMMotifSampler subroutines
