
# PMotif package
# Function: routines for evaluating motif discovery algorithms
#
# Author: Jianjun Hu (hujianju@purdue.edu)
# Kihara Laboratory of Bioinformatics
# Department of Biological Sciences
# Purdue University, West Lafayette, IN, 47906.
#
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
# 12-17-2004 
#	add the motif level accuracy and sequence/training set level accuracy
#	
# Last modified: 10-25-2004

package PMotif;
use strict;
use Descriptive;
use PToolbox;	#my collection of useful perl routines
my $debug=0;

#Seperator symbols for different level of information passed in strings:
#1st level- " " (lowest level)
#2nd level- ","
#3rd level- ":"
#4th level- ";"


#-----------------------------------------------------------
#some global declaration
#
#my  @algorithms={"AlignACE","BioProspector","MDScan","MEME","MotifSampler"};
#-----------------------------------------------------------


#FUNCTION
# 	extract target motif positions in the sequences, one for each sequence
#	(should extended to multiple target motifs for each sequence)
#INPUT:	
# 	transription factor sequence file name
# 	knownSiteFiles
#	assumption: we assume that all motifs in a sequence set are of equal length
#RETURN:
# 	an array of motif positions, each record includes 
# 	the start position of motif and its width (no limit of one motif for each sequence)

sub getMotifInfo{
    my ($filename,$knownSiteFile)=@_;
    unless(open(MOTIFFILE,$knownSiteFile)){
	print "PMotif::getMotifInfo: Cannot open file $knownSiteFile!\n";
	exit;
    }
	
    $filename=~/(.*?)\.(.*)/;
    my $geneName=$1;
    my @records=<MOTIFFILE>;
    close (MOTIFFILE);
    @records = grep {$_=~/$geneName.*/} @records;

#	relative positions of target motifs on each sequence
#	it is possible to have multiple target motifs on a single sequence
    my @motifInfo=(); #one record for all target binding sites on a sequence
    for(my  $i=0; $i<scalar @records; $i++){
		my @entries = split(/ |\t/,$records[$i]);
		my $thisMotifStartPosAbs = $entries[1];
		my $thisMotifEndPosAbs = $entries[2];
		my $motifWidth = $thisMotifEndPosAbs -$thisMotifStartPosAbs+1;
		my $motifWithMargin= $entries[4];
		push(@motifInfo,extractMotifPosWidth($motifWithMargin, $motifWidth));
    }
    return @motifInfo;
}

#compute the information content of a set of sequences
sub getInfoContent1{
	my ($array)=@_;
	my @sequences = @$array;
	my $nSequence =scalar @sequences;
	my $infoContent=0;
	
	my $minLen = 1e10;
	foreach (@sequences){
		if(length($_)<$minLen){
			$minLen = length($_);
		}
	}
	my $nPosition = $minLen;
	
	for(my $i=0;$i<$nPosition;$i++){
		my $nA=0;
		my $nC=0;
		my $nG=0;
		my $nT=0;
		my $n_=0;
		for(my $j=0;$j<$nSequence;$j++){
			my $nucleotide=substr($sequences[$j],$i,1);
			if($nucleotide eq "A"){ $nA++;}
			elsif($nucleotide eq "C"){ $nC++;}
			elsif($nucleotide eq "G"){ $nG++;}
			elsif($nucleotide eq "T"){ $nT++;}
			elsif($nucleotide eq "-"){ $n_++;}
			else{
				print "Error.getInfoContent:: illegal character in sequence [$nucleotide]\n";
				exit;
			}
		}
		my @ACGTcounts=($nA,$nC,$nG, $nT);
		my ($min,$max)=PToolbox::min_and_max(@ACGTcounts);
		my $nNucleotides = $nA+$nC+$nG+$nT;
		my @freq=($nA/$nNucleotides,$nC/$nNucleotides,$nG/$nNucleotides,$nT/$nNucleotides);
		$infoContent = 2+$freq[0]*log($freq[0])/log(2)+
				  $freq[1]*log($freq[1])/log(2)+
				  $freq[2]*log($freq[2])/log(2)+
				  $freq[3]*log($freq[3])/log(2);
		if($n_>0) {
			$infoContent=0;
		}
	}
	return $infoContent;
}

#function: return the start position of motif and its width as a string 
#          seperated by whitespace
#   input: A sequence segment with Binding site Capitalized, possible multiple binding sites
#  return:

sub extractMotifPosWidth{
    my ($sequence,$motifWidth)=@_;
    chomp $sequence;

    my @DNA=split(//,$sequence);
    my $startPos=-1;
    my $endPos=-1;
	
	#replace all low-case symbols with space. extract all the capitablized 
	#regions. Pay attention to connected multiple binding sites
	
# 	print "$sequence **************\n";
	$sequence=~s/[a-z]/ /g;
	my @sites = split(" ",$sequence);
	for (my $j=0;$j<scalar @sites;$j++){
		$sites[$j]= PToolbox::squeezeString($sites[$j]," ");
		chomp($sites[$j]);
	}
	
	#for each consecutive BINDING SITE POSITIONS, we check its length
	#if is equal to the motif width, then this is a single binding sites
	#otherwise, it is concatement of multiple sites
	my %startPosHash;

    for(my $i=0;$i<scalar @sites;$i++){
		if(length($sites[$i]) == $motifWidth){
			my @posList= PToolbox::getMatchPositions($sites[$i],$sequence);
			foreach my $startPos (@posList){
				$startPosHash{$startPos}++;
			}
		}
		else{
			my @posList= PToolbox::getMatchPositions($sites[$i],$sequence);
			my $segmentLength = length($sites[$i]);
			my $nBindingSites = $segmentLength/$motifWidth;
			foreach my $startPos (@posList){
				for(my $k=0;$k<$nBindingSites;$k++){
					$startPosHash{$startPos+$k*$motifWidth}++;
				}
			}
		}
	}
	
	my @list= keys %startPosHash;
	@list=  sort {$a <=> $b} @list;
	my $exportRecord="";
	foreach my $startPos (@list) {
  		if ($exportRecord eq ""){
 			$exportRecord = "$startPos $motifWidth";
		}
 		else{
  			$exportRecord= "$exportRecord,$startPos $motifWidth";
		}
	}
# 	print $exportRecord,"\n---------\n";
    return $exportRecord;
}


#function: given a set of sequences calculate the information content of its
#          alignment. for sequences with different length, we only count the 
#	   minimum common length from position 0
#   input: a group of sequences with equal length
#  return: information content

sub getInfoContent{
	my ($seqList)=@_;
	my $nSequences = scalar (@$seqList);
	my @sequences = @{$seqList};
	my $sumInfoContent=0;
	my @sequenceLengths=();
	for(my $i=0;$i<$nSequences;$i++){
		$sequences[$i]=uc($sequences[$i]);
		push(@sequenceLengths, length($sequences[$i]));
	}
	my ($nPosition,$max)=PToolbox::min_and_max(@sequenceLengths);
	
# 	calculate the information content of each position and sum together
#	to get the total information content of alignment
	for(my $i=0;$i<$nPosition;$i++){
		my $nA=0;
		my $nC=0;
		my $nG=0;
		my $nT=0;
		for(my $j=0;$j<$nSequences;$j++){
 			my $nucleotide=substr($sequences[$j],$i,1);
			if($nucleotide eq "A"){ $nA++;}
			elsif($nucleotide eq "C"){ $nC++;}
			elsif($nucleotide eq "G"){ $nG++;}
			elsif($nucleotide eq "T"){ $nT++;}
			else{
				print "Error in getInfoContent: [$nucleotide]\n";
			}
		}
		
		my @ACGTcounts=($nA,$nC,$nG, $nT);
		my ($min,$max)=PToolbox::min_and_max(@ACGTcounts);
		my $nNucleotides = $nA+$nC+$nG+$nT;
		my @prob=($nA/$nNucleotides,$nC/$nNucleotides,$nG/$nNucleotides,$nT/$nNucleotides);
		#print "@prob\n\n";
		my $infoContent = 0;
		my $priorProb=0.25;
		$infoContent+=$prob[0]*log($prob[0]/$priorProb)/log(2) if ($prob[0] !=0);
		$infoContent+=$prob[1]*log($prob[1]/$priorProb)/log(2) if ($prob[1] !=0);
		$infoContent+=$prob[2]*log($prob[2]/$priorProb)/log(2) if ($prob[2] !=0);
		$infoContent+=$prob[3]*log($prob[3]/$priorProb)/log(2) if ($prob[3] !=0);
		$sumInfoContent += $infoContent;
	}
	#$sumInfoContent= 2 * $nPosition + $sumInfoContent;
	$sumInfoContent = $sumInfoContent; #Hertz, 1999.
	return $sumInfoContent;
}

#function: given a group of segments in a line, divide them into clusters such that the
#		   the boundary covered by the clusters won't overlap each other and be at least
#          away from each other for the double the average segment (motif) length
#       
#		   
#          
#   input: leftBound array, rightBound array,
#  return: an array of IDs, ith item is the list of segment IDs that belong to this cluster

sub overlapCluster{
	my ($leftBound, $rightBound)=@_;
	
	#print "@$leftBound ", "==\n";
	
	#get average length of segments
	my @widths=();
	my $aveLength=0;
	for(my $i=0;$i<scalar @{$leftBound};$i++){
		$widths[$i]= ${$rightBound}[$i]-${$leftBound}[$i];
		$aveLength+=$widths[$i];
	}
	$aveLength = $aveLength/(scalar @widths);
	
	#print "aveSegLength: $aveLength \n";
	my $clusterID=0;
	my @clusterMembersList=();
	my $clusterMembers="";
	my $curClusterLeftBound=0;
	my $curClusterRightBound=0;
	for(my $i=0;$i<scalar @{$leftBound};$i++){
		my $leftPos = ${$leftBound}[$i];
		my $rightPos = ${$rightBound}[$i];
		if($clusterMembers eq ""){	#first cluster
			$clusterMembers="$i";
			$curClusterLeftBound = $leftPos;
			$curClusterRightBound = $rightPos;
		}
		else{
			if(isOverlapped($curClusterLeftBound, $curClusterRightBound,
							$leftPos, $rightPos)){
				$clusterMembers=join(' ',$clusterMembers,$i);
				$curClusterRightBound = $rightPos;
			}
			elsif( getSegmentDistance($curClusterLeftBound, $curClusterRightBound,
							$leftPos, $rightPos)<2* $aveLength){
				$clusterMembers=join(' ',$clusterMembers,$i);
				$curClusterRightBound = $rightPos;
			}
			else{#create a new cluster
				push(@clusterMembersList, $clusterMembers) if ($clusterMembers ne "");
				$clusterMembers="$i";
				$curClusterLeftBound = $leftPos;
				$curClusterRightBound = $rightPos;
			}
			if($i== scalar @{$leftBound}-1){#last record
				push(@clusterMembersList, $clusterMembers) if ($clusterMembers ne "");
			}
		}
	}
	#print "== @clusterMembersList ==\n";
	#PToolbox::printV(@clusterMembersList);
	return @clusterMembersList;
}

#check whether two segments are overlapped
sub isOverlapped{
	my ($left1, $right1, $left2, $right2)=@_;
	if($left1<$left2){
		if($right1<$left2) {
			return 0;
		}
		else{
			return 1;
		}
	}
	else{
		if($right2<$left1) {
			return 0;
		}
		else{
			return 1;
		}		
	}
}

#FUNCTION: get the distance of two segments in x axis
sub getSegmentDistance{
	my ($left1, $right1, $left2, $right2)=@_;
	if($left1<$left2){
		if($right1<$left2) {
			return $left2-$right1;
		}
		else{
			return 0;
		}
	}
	else{
		if($right2<$left1){
			return $left1-$right2;
		}
		else{
			return 0;
		}		
	}
}

#function: given a set of sequences with left and right boundaries
#          calculate their overlapping ratio
# 	   we cluster the motifs into clusters, which includes all motifs that are
# 	   connected together by overlapping at least one nucleotide positions. cluster with
# 	   the highest number of motifs will be used as the target and their consensus
# 	   will be exported as the output of the ensemble algorithms
#          
#   input: leftBound array, rightBound array,
#          threshold: ratio of consensus that is redeemed as overlap
#  return: overlap start and end positions and whole span widthn of
#          all sequences


sub mergeHistogram{
    my ($leftBound, $rightBound, $nSequences, $threshold, $estimateSites)=@_;
    #use the number of total training sequence as voters.
    #so no prediction means a negative vote, which is not good idea, maybe.

    #L: 120 25 25  how to remove noise data of this case!
    #R: 40 40 135  how to best determine the number of clusters.
    #find the cluster with maximum overlapped binding sites and remove other data
    $estimateSites = 2 if ($estimateSites ==0);
    $threshold=0.0 if($threshold ==0);
    
    my @sortLeftBound= @$leftBound;
    my @sortRightBound=@$rightBound;
	my $nPredictSites = scalar @sortLeftBound;

    $debug=0;
	#print input predictions of ensemble algorithm
	print "input:\n @sortLeftBound\n" if($debug!=0) ;
	

    my $nSegment = scalar @sortLeftBound;
    if($nSegment ==1){
	    #print "surprise:  $sortLeftBound[0], $sortRightBound[0].........\n";
	    return ($sortLeftBound[0], $sortRightBound[0], $sortRightBound[0]-$sortLeftBound[0]+1);
    }
    elsif ($nSegment ==0){	#TODO BUG?  should not happen!
	    return(0,0,0);
    }
    #PToolbox::printV(@sortLeftBound);

    #calculate average predicted site widths
    my $aveMotifWidth=0;
    for(my $j=0;$j<scalar @sortLeftBound;$j++){
		#print "**mwidth", $sortRightBound[$j]-$sortLeftBound[$j]+1,"\n";
		$aveMotifWidth =$aveMotifWidth+ $sortRightBound[$j]-$sortLeftBound[$j]+1;
    }
    $aveMotifWidth =$aveMotifWidth/$nPredictSites;

    #the ensemble assembling algorithm begins here
    #identifying overlap region with above threshod agreement and the
    #predicted motif widths will be no longer than max(20, maxPredictedMotifWidthI), otherwise, the
    #ensemble algorithm will be sensitive, but lack of specificity
    #my $maxMotifLength=20;	#default value, PMDEnsemble will only predict 20 nucleotides motifs
    my ($minMotifLength,$maxMotifLength) =PToolbox::min_and_max(20,$aveMotifWidth);

    #remove some bad predictions with -1 as the label
    for(my $i=$nPredictSites-1;$i>=0;$i--){
		if($$leftBound[$i]==-1 ||$$rightBound[$i]==-1){
			splice(@sortLeftBound,$i,1);
			splice(@sortRightBound,$i,1);
		}
    }
    my @voteLeftBound=@sortLeftBound;
    my @voteRightBound=@sortRightBound;
    
    my $nVoters=scalar @sortLeftBound;
    
    @sortLeftBound= sort {$a <=> $b} (@sortLeftBound);
    @sortRightBound=sort {$b <=> $a} (@sortRightBound);

    my $leftMost = $sortLeftBound[0];
    my $rightMost= $sortRightBound[0];
    
    #$count>=$threshold * $nVoters;
    #squeeze or expand the predictions to the average predicted motif widths

    
    #count the number of votes for all the positions between $leftMost and $rightMost
    my @counts=();
    for(my $i=$leftMost;$i<=$rightMost;$i++){
		$counts[$i]=0;
    }
    for(my $i=$leftMost;$i<=$rightMost;$i++){
		for(my $j=0;$j<$nPredictSites;$j++){
			if(isOverlapped($i,$i, $voteLeftBound[$j],$voteRightBound[$j])){
			$counts[$i]++;
			}
		}
    }
	
	#sliding average of the votes to remove small peaks March30,05
	my $winSize = int($aveMotifWidth*1);#originally 20
	my @countsAve=();
	for(my $i=$leftMost;$i<=$rightMost;$i++){
		my $winLeft 	= int($i-$winSize/2);
		my $winRight 	= int($i+$winSize/2+0.5);
		my $votesTotal 	=0;
		for(my $j=$winLeft;$j<=$winRight;$j++){
			 if($j>=$leftMost && $j<=$rightMost){
				$votesTotal += $counts[$j];
			 }
		}
		$countsAve[$i]=$votesTotal;
	}	
	@counts = @countsAve;
	
    PToolbox::printH(@counts) if($debug !=0);
	
    #find all the peaks of counts and then average to get the center of motif
    #then we expand in exither direction climbing the counts
    my $direction=-1;
    my $lastCounts=-1;
    my $curPeak=0;
    my $peakLeft=0;
    my $peakRight=0;
    my @peakList=();
    for(my $i=$leftMost;$i<=$rightMost;$i++){
	my $newDirection=0;
	if($counts[$i]>$lastCounts){
	    $peakLeft=$i;
	    $peakRight=$i;
	    $direction=1;
	    $lastCounts=$counts[$i];
	}
	elsif($counts[$i] == $lastCounts){	#we set the peak in the middle of flat region
	    $peakRight=$i;
	    #$direction=0;
	    if($i==$rightMost and $direction>0){#last one
		$curPeak=int(($peakLeft+$peakRight)/2);
		 push(@peakList,$curPeak);
	    }
	}
	else{
	    $curPeak=int(($peakLeft+$peakRight)/2);
	    if($direction>=0){ #this is a turnaround point, save this peak
		push(@peakList,$curPeak);
	    }
	    $direction=-1;
	    $lastCounts=$counts[$i];
	    $peakLeft=$i;
	    $peakRight=$i;
	}
    }
    PToolbox::printH(@peakList) if($debug ==1);
	
    #how many peaks to report as binding sites?
    #firs screen out the peak positions whose votes are greater than the threshold
    my @candiatePeakIndex=();
    for(my $i=0;$i<scalar @peakList;$i++){
		#print $threshold, "  ",$counts[$peakList[$i]], "  ",$nSequences*$threshold, "***\n";
		if($counts[$peakList[$i]]>= $threshold){
			#use threshold to control the number of site predictions
			push(@candiatePeakIndex,$i);
		}
    }
	
	print "estimateSites = $estimateSites\n" if($debug !=0);
	if(scalar @candiatePeakIndex < $estimateSites){
		$estimateSites= scalar @candiatePeakIndex;
	}
	#we only use threshold to control Num of sites to report
	
    if(scalar @candiatePeakIndex ==0){
		#print "exit................\n";
		return (0, 0, 0);
    }
    #pick the top $estimateSites peaks as the predictions
    #sort the index accoriding to its vote numbers
    my @siteCenters=();    
    if(scalar @peakList <=$estimateSites){
		@siteCenters = @peakList;
    }
    else{
		#descending order sorting, only need to swithc $a and $b
		@candiatePeakIndex = sort { $counts[$peakList[$b]] <=> $counts[$peakList[$a]]} @candiatePeakIndex;
		
		if($debug!=0){
			foreach (@candiatePeakIndex){
				print $counts[$peakList[$_]], "\n";
			}
		}
		
		for(my $i=0;$i<$estimateSites;$i++){
			push(@siteCenters, $peakList[$candiatePeakIndex[$i]]);
		}	
    }
    
	PToolbox::printH(@siteCenters) if ($debug !=0);
	
    #for each peak as the predicted center, we extend it to generate a site prediction
    my $motifLeft=0;
    my $motifRight=0;
    my @predictList=();
	my @predictMotifLeft=();
    foreach (@siteCenters){
		my $motifCenter=$_;
		my $bestCount=0;
		#extension in both direction
		$motifLeft=$motifCenter;
		$motifRight=$motifCenter;
		while($motifRight-$motifLeft+1< $minMotifLength){
			if($counts[$motifLeft-1]>$counts[$motifRight+1]){
			$motifLeft=$motifLeft-1;
			}
			elsif($counts[$motifLeft-1] == $counts[$motifRight+1]){
			if(rand(1)<0.5){
				$motifLeft=$motifLeft-1;
			}
			else{
				$motifRight=$motifRight+1;
			}
			}
			else{
			$motifRight=$motifRight+1;
			}
		}
		my $motifwidth=$motifRight-$motifLeft+1;
		push(@predictList, "$motifLeft $motifwidth");
		push(@predictMotifLeft, $motifLeft);
    }
	
	#print output of ensemble algorithm
	print "result\n @predictMotifLeft\n" if($debug !=0);
    return @predictList;
}

#------------------------------------------------------------------------
#get the overlapped predictions determined by the threshold of agreement
sub getOverlapInfo{
    my ($leftBound, $rightBound, $nSequences, $threshold)=@_;
    #use the number of total training sequence as voters.
    #so no prediction means a negative vote, which is not good idea, maybe.

	#L: 120 25 25  how to remove noise data of this case!
	#R: 40 40 135  how to best determine the number of clusters.
	#find the cluster with maximum overlapped binding sites and remove other data
	print "Error in PMotif::getOverlapInfo: nSequence must be set..\n" if($nSequences==0);
	
    my @sortLeftBound= @$leftBound;
    my @sortRightBound=@$rightBound;
	my $nPredictSites = scalar @sortLeftBound;
	
	my $nSegment = scalar @sortLeftBound;
	if($nSegment ==1){
		#print "surprise:  $sortLeftBound[0], $sortRightBound[0].........\n";
		return ($sortLeftBound[0], $sortRightBound[0], $sortRightBound[0]-$sortLeftBound[0]+1);
	}
	elsif ($nSegment ==0){	#TODO BUG?  should not happen!
		return(0,0,0);
	}
	
	#find the cluster with maximum number of members
	#PToolbox::printV(@sortLeftBound);
	
	my @clusterList=overlapCluster(\@sortLeftBound, \@sortRightBound);
	#print scalar @clusterList, "***\n";
	my $maxClusterID=0;
	my $maxClusterMemberNo=0;
	for(my $i=0;$i<scalar @clusterList;$i++){
		my @items = split(' ', $clusterList[$i]);
		if(scalar @items > $maxClusterMemberNo){
			$maxClusterMemberNo = scalar @items;
			$maxClusterID = $i;
		}
	}
	my @clusterMembers = split(' ',$clusterList[$maxClusterID]);
	#extract the new groups of boundaries
	my @newLeftBounds=();
	my @newRightBounds=();
	foreach (@clusterMembers){
		push(@newLeftBounds, $sortLeftBound[$_]);
		push(@newRightBounds, $sortRightBound[$_]);
	}
	@sortLeftBound = @newLeftBounds;
	@sortRightBound = @newRightBounds;
	#PToolbox::printH(@sortLeftBound);
		
	
	#calculate average predicted site widths
	my $aveMotifWidth=0;
	for(my $j=0;$j<scalar @sortLeftBound;$j++){
	    $aveMotifWidth =$aveMotifWidth+ $sortRightBound[$j]-$sortLeftBound[$j]+1;
	}
	$aveMotifWidth =$aveMotifWidth/$nPredictSites;
	
	#the ensemble assembling algorithm begins here
	#identifying overlap region with above threshod agreement and the
	#predicted motif widths will be no longer than max(20, maxPredictedMotifWidthI), otherwise, the
	#ensemble algorithm will be sensitive, but lack of specificity
	#my $maxMotifLength=20;	#default value, PMDEnsemble will only predict 20 nucleotides motifs
	my ($minMotifLength,$maxMotifLength) =PToolbox::min_and_max(20,$aveMotifWidth);
	
    for(my $i=$nSequences-1;$i>=0;$i--){
		if($$leftBound[$i]==-1 ||$$rightBound[$i]==-1){
			splice(@sortLeftBound,$i,1);
			splice(@sortRightBound,$i,1);
		}
    }
    my @voteLeftBound=@sortLeftBound;
    my @voteRightBound=@sortRightBound;
    my $nVoters=scalar @sortLeftBound;
    
    @sortLeftBound= sort {$a <=> $b} (@sortLeftBound);
    @sortRightBound=sort {$a <=> $b} (@sortRightBound);
    @sortRightBound = reverse(@sortRightBound); #from max to min

    my $leftMost = $sortLeftBound[0];
    my $rightMost= $sortRightBound[0];
    my $spanWidth = $rightMost-$leftMost+1;
    if($rightMost == $leftMost){
		$spanWidth=0;
    }
    my $overlapStart=0;
    my $overlapEnd=0;
	
    for(my $i=0;$i<scalar @sortLeftBound; $i++){
	my $count=0;
	for(my $j=0;$j< scalar @voteLeftBound; $j++){
	    if( $voteLeftBound[$j]<=$sortLeftBound[$i] && $voteRightBound[$j]>=$sortLeftBound[$i]){
		$count++;
	    }
	}
	if($count>=$threshold * $nVoters){
	    $overlapStart = $sortLeftBound[$i];
	    last;
	}
    }

    for(my $i=0;$i<scalar @sortRightBound; $i++){
		my $count=0;
		for(my $j=0;$j< scalar @voteRightBound; $j++){
			if( $voteLeftBound[$j]<=$sortRightBound[$i] && $voteRightBound[$j]>=$sortRightBound[$i]){
			$count++;
			}
		}
	
		if($count>=$threshold * $nVoters){
			$overlapEnd = $sortRightBound[$i];
			last;
		}
    }
    #print "output: $overlapStart   $overlapEnd\n\n";
    my $overlapLength=0;
    if($overlapEnd>$overlapStart){
		$overlapLength = $overlapEnd - $overlapStart+1;
    }
    else{
		$overlapLength = 0;
    }
    
    #squeeze or expand the predictions to the average predicted motif widths
    my $motifLeft=0;
    my $motifRight=0;
    if($overlapLength != $minMotifLength){
	my @counts=();
	for(my $i=0;$i<=$rightMost;$i++){
	    $counts[$i]=0;
	}
	for(my $i=$leftMost;$i<=$rightMost;$i++){
	    for(my $j=0;$j<$nSequences;$j++){
		if(isOverlapped($i,$i, $voteLeftBound[$j],$voteRightBound[$j])){
		    $counts[$i]++;
		}
	    }
	}
	#PToolbox::printH(@counts);
	#print $minMotifLength, "****\n";
	
	#find all the peaks of counts and then average to get the center of motif
	#then we expand in exither direction climbing the counts
	my $direction=-1;
	my $lastCounts=-1;
	my $curPeak=0;
	my $peakLeft=0;
	my $peakRight=0;
	my @peakList=();
	for(my $i=$overlapStart;$i<=$overlapEnd;$i++){
	    my $newDirection=0;
	    if($counts[$i]>$lastCounts){
		$peakLeft=$i;
		$peakRight=$i;
		$direction=1;
		$lastCounts=$counts[$i];
	    }
	    elsif($counts[$i] == $lastCounts){	#we set the peak in the middle of flat region
		$peakRight=$i;
		#$direction=0;
		if($i==$overlapEnd and $direction>0){#last one
		    $curPeak=int(($peakLeft+$peakRight)/2);
		     push(@peakList,$curPeak);
		}
	    }
	    else{
		$curPeak=int(($peakLeft+$peakRight)/2);
		if($direction>=0){ #this is a turnaround point, save this peak
		    push(@peakList,$curPeak);
		}
		$direction=-1;
		$lastCounts=$counts[$i];
		$peakLeft=$i;
		$peakRight=$i;
	    }
	}
	#PToolbox::printH(@peakList);
	
	#method 1: use average peak, not good
	#my $motifCenter=0;
	#foreach(@peakList){
	#    $motifCenter+= $_;
	#}
	#$motifCenter=int($motifCenter/(scalar @peakList));
	
	#method2: use the peak with highest counts as the motif center
	my $motifCenter=0;
	my $bestCount=0;
	for(my $i=0;$i<scalar @peakList;$i++){
	    if($counts[$peakList[$i]]>$bestCount){
		$bestCount=$counts[$peakList[$i]];
		$motifCenter=$peakList[$i];
	    }
	}	
	
	#extension in both direction
	$motifLeft=$motifCenter;
	$motifRight=$motifCenter;
	while($motifRight-$motifLeft+1< $minMotifLength){
	    if($counts[$motifLeft-1]>$counts[$motifRight+1]){
		$motifLeft=$motifLeft-1;
	    }
	    elsif($counts[$motifLeft-1] == $counts[$motifRight+1]){
		if(rand(1)<0.5){
		    $motifLeft=$motifLeft-1;
		}
		else{
		    $motifRight=$motifRight+1;
		}
	    }
	    else{
		$motifRight=$motifRight+1;
	    }
	}
	print "*** $motifLeft   $motifRight\n\n";
    }
    #method 3:
    #comment out this to don't do expansion or squeee to improve the specificity
    ($overlapStart, $overlapEnd)=($motifLeft,$motifRight);
    if($spanWidth<0){
    	print "L: @sortLeftBound\n\n";
		print "R: @sortRightBound\n\n";
    }
    return ($overlapStart, $overlapEnd, $spanWidth);
}

#------------------------------------------------------------------------

#report only the TOP $nScore accuracy score according to the actural accuracy
sub getTopScores{
    my($nScore,$accuracyScores)=@_;
    #report only the TOP $nMotif accuracy score
    my %hashScores=();

    if($nScore > scalar @$accuracyScores){
        $nScore=scalar @$accuracyScores;
    }
    if($nScore!=0){
        my @topScores=();
        #screen out best K scores by sorting
        #create a hash and then sort
        for(my $i=0;$i<scalar @$accuracyScores;$i++){
            my @items=split(' ',@$accuracyScores[$i]);
	    	#TFname accuracy sensitivity specifificty score nSequence motifWidth
            $hashScores{$i}=$items[1];#sort by score
        }

        #sort hash by the score, get the top indexes with top score
        my @topKeys = sort { $hashScores{$b} <=> $hashScores{$a} } keys %hashScores;

        #print "\n--",$topKeys[0],"\n\n";
        for(my $i=0;$i<$nScore;$i++){
            push(@topScores,@$accuracyScores[$topKeys[$i]]);
        }
        return @topScores;
    }
    else{
		return @$accuracyScores;
    }
}

#get specified score statistics for one or more top motifs
sub getTopScoreStat{
    my($nScore,$accuracyScores, $stat)=@_;
	
	my @topScores = ();
	my @accuracyScoreList = @$accuracyScores;
	if(scalar @accuracyScoreList ==0 || $nScore ==0){
		return @topScores;
	}
	if ($stat eq "MEAN" || $stat eq "STDDEV"){
		my @items=split(' ',$accuracyScoreList[0]);
		my $TF=$items[0];
		my $nNumbers = scalar @items -1;
		
		my @sumItems=();
		my $statUnit = Statistics::Descriptive::Full->new();
		for(my $i=1;$i<=$nNumbers;$i++){
			my @data=();
			for(my $j=0;$j<scalar @accuracyScoreList;$j++){
				@items=split(' ',$accuracyScoreList[$j]);
				push(@data, $items[$i]);
			}
			$statUnit->add_data(@data);
			if ($stat eq "MEAN"){
				push(@sumItems, $statUnit->mean());
			}
			elsif ($stat eq "STDDEV"){
				push(@sumItems, $statUnit->standard_deviation());
			}
		}
		push(@topScores, join(" ", $TF,@sumItems));
		return @topScores;
	}
	else{
		#report only the TOP $nMotif accuracy score
		my %hashScores=();
		if($nScore > scalar @accuracyScoreList){
			$nScore=scalar @accuracyScoreList;
		}
		#screen out best K scores by sorting
		#create a hash and then sort
		for(my $i=0;$i<scalar @accuracyScoreList;$i++){
			my @items=split(' ',$accuracyScoreList[$i]);
			#TFname accuracy sensitivity specifificty score nSequence motifWidth
			$hashScores{$i}=$items[1];#sort by accuracy
		}
		#sort hash by the score, get the top indexes with top score
		my @topKeys = ();
		#exporting specific statistic accuracy
		if($stat eq "WORST"){
			@topKeys=sort { $hashScores{$a} <=> $hashScores{$b} } keys %hashScores;
		}
		elsif($stat eq "BEST"){
			@topKeys=sort { $hashScores{$b} <=> $hashScores{$a} } keys %hashScores;
		}
		push(@topScores,$accuracyScoreList[$topKeys[$0]]);
		return @topScores;
	}
}


#function: calculate accuracy based on overlap region
#          
#   input: start and end positions of predicted and experimental motifs
#          startPos endPos   predicted motif positions
#          startPos1 endPos1 expeirment motif positions 
#  return: an accuracy score

sub getAccuracyScore{
    my ($startPos,$endPos,$startPos1,$endPos1)=@_;
    
    my $accuracy=0;
    my $sensitivity=0;#TP/(TP+FN)
    my $specificity=0;#TP/(TP+FP)

    my $overlapStartPos;
    my $overlapEndPos;
    my $leftMost;
    my $rightMost;
    
    my $width=$endPos-$startPos+1; #predict motif width
    my $width1=$endPos1-$startPos1+1; #experimental motif width

    if($startPos1>=$startPos){
	$overlapStartPos=$startPos1;
	$leftMost=$startPos;
    }
    else{
	$overlapStartPos=$startPos;
	$leftMost=$startPos1;
    }
    
    if($endPos1>=$endPos){
	$overlapEndPos=$endPos;
	$rightMost=$endPos1;
    }
    else{
	$overlapEndPos=$endPos1;
	$rightMost=$endPos;
    }
    #print "$overlapStartPos  $overlapEndPos\n";
    my $spannedWidth=$rightMost-$leftMost+1;

    if($overlapStartPos>$overlapEndPos){
	$accuracy=0;
	$specificity=0;
	$sensitivity=0;
    }
    else{
	#method 1
	#$accuracy=($overlapEndPos-$overlapStartPos)**2/($width*$width1);
	#method 2, others use this
	#print "--$spannedWidth $startPos,$endPos,$startPos1,$endPos1 ***\n";
	if($spannedWidth==0){
	    $accuracy=0;
	    $specificity=0;
	    $sensitivity=0;
	}
	else{
	    my $overlapLen =$overlapEndPos-$overlapStartPos+1;
	    $accuracy= $overlapLen/$spannedWidth;	#TP/(TP+FN+FP)
	    $sensitivity=$overlapLen/$width1;		#TP/(TP+FN)
	    if($width==0){
		$specificity=0;
	    }else{
		$specificity=$overlapLen/$width;	#TP/(TP+FP)
	    }

	    #print "overlap= ",$overlapLen," spanwid=",$spannedWidth,"\n";
	    #we can define other criteria as performance measure like sensitivity,specificity,etc
	    #calculate TP, FN, FP, TN. then we can calculate many evaluation scores
	}
    }
    #print "$startPos,$endPos,$startPos1,$endPos1,\n";
    #print "$accuracy,$sensitivity, $specificity \n";
    return ($accuracy,$sensitivity, $specificity);
}

sub getNofDuplicates{
	my ($seqIDList) =@_;
	my @nonDuplicateSeqIDs=();
	my %seen=();
	foreach (@$seqIDList) {
    		push(@nonDuplicateSeqIDs, $_) unless ($seen{$_}++);
	}
	return scalar @$seqIDList -scalar @nonDuplicateSeqIDs;
}

# get_file_data
#
# A subroutine to get data from a file given its filename

sub get_file_data {
    my($filename) = @_;
    my @filedata = (  );

    unless( open(GET_FILE_DATA, $filename) ) {
        print STDERR "Cannot open file \"$filename\"\n\n";
        exit;
    }

    @filedata = <GET_FILE_DATA>;
    close GET_FILE_DATA;
    return @filedata;
}

#show the statistics of an array of numbers
sub showStats{
    my @data=@_;
    my ($stat);
    my ($num);
    my ($average);
    my ($variance);
    my ($std_dev);

  #defines an object
    $stat = Statistics::Descriptive::Full->new();
  
  #Adds data to the cached statistics values
    $stat->add_data(@data);
  #
    $num=$stat->count();
    $average = $stat->mean();
    $variance =$stat->variance(); 
    $std_dev = $stat->standard_deviation();

  print  "=========================================\n",
        "Num. of measurments = $num\n",
        "Average             = $average\n",
        "Variance            = $variance\n",
    "Standard Deviation  = $std_dev\n\n"; 
}




#function: get a sequence with motif capitalized and margin nucleotides lower cased.
#input: a sequence with all capitalized nucleotides, 
#       startPos of real motif in genome
#       startPos of this intput sequence in genome
#       motif length
#used in genMotifData.pl

sub getKnownSiteInSequence{
	my($motifRecordNew,$startPos,$startPos1,$motifLength)=@_;
	#print $motifRecordNew,"  ",$startPos,"  ",$startPos1,"  ",$motifLength,"\n";
	my $offsetLeft=$startPos-$startPos1-1;
	my $leftMargin=lc(substr($motifRecordNew,0,$offsetLeft));
	#print $offsetLeft,"  ",$offsetLeft + $motifLength,"  ",length($motifRecordNew),"\n";
	if($offsetLeft + $motifLength>length($motifRecordNew)){
		$motifLength=length($motifLength-$offsetLeft);
	}
	
	my $motif=substr($motifRecordNew,$offsetLeft,$motifLength);
	my $rightMargin=lc(substr($motifRecordNew,$offsetLeft+$motifLength));
	return "$leftMargin$motif$rightMargin";
}

#function: get a sequence with motif capitalized and margin nucleotides lower cased.
#input: a sequence with all capitalized nucleotides, 
#       startPos of real motif in genome
#       startPos of this intput sequence in genome
#       motif length
#not used yet in genMotifDataX.pl

sub getKnownSiteInSequenceX{
	my($motifRecordNew,$startPos,$startPos1,$motifLength)=@_;
	my $offsetLeft=$startPos-$startPos1-1;
	my $leftMargin=lc(substr($motifRecordNew,0,$offsetLeft));
	if($offsetLeft + $motifLength>length($motifRecordNew)){
		$motifLength=length($motifLength-$offsetLeft);
	}
	my $motif=substr($motifRecordNew,$offsetLeft,$motifLength);
	my $rightMargin=lc(substr($motifRecordNew,$offsetLeft+$motifLength));
	return "$leftMargin$motif$rightMargin";
}


#found the gene boundary given the boundary of a motif
#input: geneLefts--left boundary positions of all genes in genome
#       geneRights--right boundary positions of all genes in genome
#       leftMotifB--left boundary position of a given motif
#       rightMotifB--right boundary position of a given motif

sub getGeneBoundary{
    my ($geneLefts, $geneRights, $leftMotifB, $rightMotifB)=@_;
    my @leftBoundaryList=@$geneLefts;
    my @rightBoundaryList=@$geneRights;

    my $leftB=0;
    my $rightB=0;
    my $i=0;
    for( $i=0;$i<scalar @leftBoundaryList;$i++){
	if($rightBoundaryList[$i]>=$rightMotifB){
	    last;
	}
    }
    $leftB=$rightBoundaryList[$i-1];
    $rightB=$leftBoundaryList[$i];

    #print "$leftB $leftMotifB $rightMotifB $rightB ",$rightB-$leftB, "\n";
    return ($leftB, $rightB);

}

#---------------------------------------------------------------------------
#------------------------------------------------------------------------
#function: calculate the accuracy for all motifs in an output file of AlignACE
#input:    a filename of alignACE output
#return:   accuracy values for the discovered motifs along with its statistical scores
#          $bindingTF\t$aveAccuracy\t$score\t$nSeq\t$motifWidth"
#limitation:  this procedure is good for one target motif per sequence 
#			it could however handle multiple predictions for each sequence

sub getAccuracyAve{
	#a good way to handle variable number of arguments and default values
	my ($filename,$nMotif,$nScore,$algorithm)=@_;
	$nMotif=1 if $nMotif ==0; #number of motifs to be reported
	$nScore=0 if $nScore ==0; #number of top scores to report among the reported motifs,0-all,k-given no.
	$algorithm="AlignACE" if $algorithm eq "";
	my $stat = "BEST"; #BEST, MEAN, WORST, STDDEV,
						 #TOPSCORED is calculated by sumMotifAve.pl -n1 -t1
    
    my $srcFolder= "./step6$algorithm";
    if($nScore>$nMotif && $nMotif!=0){
	print "Error PMotif::getAccuracy: nScore($nScore) should be smaller than nMotif ($nMotif)\n";
	exit;
    }

    my @accuracyScores=();#store the average accuracy of all motifs discovered by the algorithm

    #load the experimental motif data
    my $knownSiteFile="./knownsites.txt";
    my @motifsExp =  PMotif::getMotifInfo($filename,$knownSiteFile);
	        
    $filename=~/(.*)\..*/;
    my $bindingTF = $1;

    #extract K  predicted binding sites info
    my @motifsPredict = ();
    #PMAlignACE::parseMotifInfo("./step6AlignACE/$filename",$nMotif);
    if($algorithm eq "AlignACE")
    {@motifsPredict = PMAlignACE::parseMotifInfo("$srcFolder/$filename",$nMotif);}
    elsif ($algorithm eq "MEME")
    {@motifsPredict = PMMeme::parseMotifInfo("$srcFolder/$filename",$nMotif);}
    elsif($algorithm eq "MDScan")
    {@motifsPredict = PMMDScan::parseMotifInfo("$srcFolder/$filename",$nMotif);}
    elsif($algorithm eq "MotifSampler")
    {@motifsPredict = PMMotifSampler::parseMotifInfo("$srcFolder/$filename",$nMotif);}
    elsif($algorithm eq "BioProspector")
    {@motifsPredict = PMBioProspector::parseMotifInfo("$srcFolder/$filename",$nMotif);}
    else {print "PMotif::getConvergeAveAccuracy: I don't recognize this algorithm $algorithm\n";}
    
    my $motifWidth=0;
    #print "****** returned motif ",scalar @motifsPredict,"   $nMotif\n";
    my $nSeq= scalar @motifsExp;

    #calculate accurary
    foreach my $record (@motifsPredict){ #each record is one group of predictions of binding sites
		my ($itemList,$score)=split(':',$record);
		my @items =  split(',',$itemList);
		my @accuracyList=();
		my @sensitivityList=();
		my @specificityList=();
	
		my @seqIDList=();
		my %nHitSeqID=(); 			#number of hits on  target sites for each sequence
		my %nTargetHitSeqID=(); 	#number of targets being hit by prediction for a sequence
		my %nPredictionSeqID=();	#number of predictions on each sequence
		my %seq2accuracyMap=(); 	#map sequenceID to accuracy scores
		my %seq2sensitivityMap=(); 	#map sequenceID to accuracy scores
		my %seq2specificityMap=(); 	#map sequenceID to accuracy scores
	
		my $nSequences = scalar @motifsExp;
		for(my $i=0;$i<$nSequences;$i++){
			$nHitSeqID{$i}=0;
			$nPredictionSeqID{$i}=0;
			$seq2accuracyMap{$i}=0;
			$seq2sensitivityMap{$i}=0;
			$seq2specificityMap{$i}=0;
		}
	
		foreach my $item (@items){#for prediction site on a sequence
			my ($seqID, $startPos, $width)=split(' ',$item);
			my $endPos=$startPos+$width-1;
			
			#handling multiple binding sites on a sequence
			#find the closest target binding site for the predicted one to calcualte
			#the necleotide level accuracy
			my @startPosPairs = split(',',$motifsExp[$seqID]); 
			my $accuracyMax=0; my $sensitivityMax=0; my $specificityMax=0;
			my $accuracy=0; my $sensitivity=0; my $specificity=0;
			
			#find the maximally overlapped target site
			for(my $i=0;$i<scalar @startPosPairs;$i++){
				my ($startPos1,$width1)=split(' ',$startPosPairs[$i]);
				my $endPos1=$startPos1+$width1-1;
				$motifWidth=$width1;
				my ($accuracy,$sensitivity,$specificity)=
					PMotif::getAccuracyScore($startPos,$endPos,$startPos1,$endPos1);
				if($accuracy>$accuracyMax){
					$accuracyMax = $accuracy;
					$sensitivityMax= $sensitivity;
					$specificityMax= $specificity;
				}
			}
			$accuracy = $accuracyMax;
			$sensitivity= $sensitivityMax;
			$specificity= $specificityMax;
			
			$seq2accuracyMap{$seqID} +=$accuracy;
			$seq2sensitivityMap{$seqID} +=$sensitivity;
			$seq2specificityMap{$seqID} +=$specificity;
			$nPredictionSeqID{$seqID}++;
			if($accuracy !=0){
				if($nHitSeqID{$seqID} ==0) {
					$nTargetHitSeqID{$seqID}++;
				}
				$nHitSeqID{$seqID}++;
			}
		}
	
		#compute binding site level accuracy (handling multiple binding sites on one sequence)
		my $accuracySite=0;
		my $sensitivitySite=0;
		my $specificitySite=0;
		for(my $i=0;$i<$nSequences;$i++){
			my @startPosPairs = split(',',$motifsExp[$i]);
			my $nTargetOfSeq = scalar @startPosPairs;
			my $TP = $nTargetHitSeqID{$i};
			my $FN = $nTargetOfSeq - $TP;
			my $FP = $nPredictionSeqID{$i}-$nHitSeqID{$i};
						
			if($TP!=0){
				$accuracySite += $TP/($FN+$TP+$FP);
				$sensitivitySite += $TP/($FN+$TP);
				$specificitySite += $TP/($TP+$FP);
			}
#  			print "$TP, $FN, $FP, \n";
		}
		$accuracySite = $accuracySite/$nSequences;
		$sensitivitySite = $sensitivitySite/$nSequences;
		$specificitySite = $specificitySite/$nSequences;
		

		#compute nucleotide level accurcy scores (handling multiple binding sites on one sequence)
		#count how many overlapping prediction-target pairs and non-overlapping pairs
		my $aveAccuracy;
		my $aveSensitivity;
		my $aveSpecificity;	
		for(my $i=0;$i<$nSequences;$i++){
			my @startPosPairs = split(',',$motifsExp[$i]);
			my $nTargetOfSeq = scalar @startPosPairs;
			
			#count prediction pairs including non-overlapping pairs
			my $nTargetPredictPairs =$nHitSeqID{$i};#matched pairs
			my $nMissedPredict= $nPredictionSeqID{$i}-$nHitSeqID{$i};
			my $nMissedTarget=$nTargetOfSeq-$nTargetHitSeqID{$i};
			if($nMissedPredict>=$nMissedTarget){
				$nTargetPredictPairs +=$nMissedPredict;
			}
			else{
				$nTargetPredictPairs +=$nMissedTarget;
			}
			
			#for each umpaired motifs, assume their accuracy scores are all 0
			$seq2accuracyMap{$i} /=$nTargetPredictPairs;	
			$seq2sensitivityMap{$i} /=$nTargetPredictPairs;
			$seq2specificityMap{$i} /=$nTargetPredictPairs;
			$aveAccuracy += $seq2accuracyMap{$i};
			$aveSensitivity +=$seq2sensitivityMap{$i};
			$aveSpecificity +=$seq2specificityMap{$i};
		}
		$aveAccuracy /= $nSequences;
		$aveSensitivity /= $nSequences;
		$aveSpecificity /= $nSequences;
	
		push(@accuracyScores, "$bindingTF\t$aveAccuracy\t$aveSensitivity\t$aveSpecificity\t$score\t$motifWidth\t$nSeq\t$accuracySite\t$sensitivitySite\t$specificitySite");                                         
    }
	
    #in case AlignACE doesn't predict any motif
    if(scalar @accuracyScores==0){
    		if($motifWidth==0){
			my ($startPos1,$width1)=split(' ',$motifsExp[0]); #assume equal motif width for all sequences
			$motifWidth = $width1;
		}
		push(@accuracyScores,"$bindingTF\t0.0\t0.0\t0.0\t0.0\t$motifWidth\t$nSeq\t0\t0\t0");
    }
    
    #report only the TOP $nMotif accuracy score (scored by accuracy)
    #print "nscore==$nScore.........\n";s
    if($nScore!=0){

		#my @topScores=PMotif::getTopScores($nScore,\@accuracyScores);
		my @topScores=PMotif::getTopScoreStat($nScore,\@accuracyScores, $stat);
		return @topScores;
    }
    else{
		return @accuracyScores;
    }
}


#------------------------------------------------------------------------
#function: calculate the accuracy for each predicted site and its most closely matched target site
#input:    a filename of alignACE output
#return:   accuracy values for the discovered motifs along with its statistical scores
#          $bindingTF\t$aveAccuracy\t$score\t$nSeq\t$motifWidth"
sub getAccuracy{
	#a good way to handle variable number of arguments and default values
	my ($filename,$nMotif,$nScore,$algorithm)=@_;
	$nMotif=1 if $nMotif ==0; #number of motifs to be reported
	$nScore=0 if $nScore ==0; #number of top scores to report among the reported motifs,0-all,k-given no.
	$algorithm="AlignACE" if $algorithm eq "";
    
	if($nScore>$nMotif && $nMotif!=0){
	print "Error PMotif::getAccuracy: nScore($nScore) should be smaller than nMotif ($nMotif)\n";
	exit;
    }
    
	my $srcFolder= "./step6$algorithm";
    my @accuracyScores=();#store the average accuracy of all motifs discovered by the algorithm
    #load the experimental motif data
    my $knownSiteFile="./knownsites.txt";
    my @motifsExp =  PMotif::getMotifInfo($filename,$knownSiteFile);
    
    $filename=~/(.*)\..*/;
    my $bindingTF = $1;

    #extract K  predicted motifs info
    my @motifsPredict = ();
    if($algorithm eq "AlignACE")
    {@motifsPredict = PMAlignACE::parseMotifInfo("$srcFolder/$filename",$nMotif);}
    elsif ($algorithm eq "MEME")
    {@motifsPredict = PMMeme::parseMotifInfo("$srcFolder/$filename",$nMotif);}
    elsif($algorithm eq "MDScan")
    {@motifsPredict = PMMDScan::parseMotifInfo("$srcFolder/$filename",$nMotif);}
    elsif($algorithm eq "MotifSampler")
    {@motifsPredict = PMMotifSampler::parseMotifInfo("$srcFolder/$filename",$nMotif);}
    elsif($algorithm eq "BioProspector")
    {@motifsPredict = PMBioProspector::parseMotifInfo("$srcFolder/$filename",$nMotif);}
    else {print "PMotif::getConvergeAveAccuracy: I don't recognize this algorithm $algorithm\n";}
	
    my $motifWidth=0;
    my $nSeq= scalar @motifsExp;
    my @seqAccuracyScores=();#store the average accuracy of all motifs discovered by the algorithm
    my $score=0;
    my @motifAccuracyScores=();
    
    #calculate accurary for each group of motif prediction
	foreach my $record (@motifsPredict){
		#merge the predicted sites of each sequence together
		
		#for all predicted sites of a sequence, calcualte its average nucleotide leve accuracy
		#for each predict-target pair. Average together to get average accuracyN over sequence
		#and then average over a data set. (consider uncovered target sites as accuracyN=0).
		my ($itemList,$score)=split(':',$record);
		my @items =  split(',',$itemList);#each item contains one predicted sites of a sequence
		my $aveAccuracy=0;
		my @seqIDList=();
		foreach my $item (@items){	#each item contains one predicted sites of a sequence
			my ($seqID, $startPos, $width)=split(' ',$item);
			push(@seqIDList,$seqID);
			my $endPos=$startPos+$width-1;
			
			#check the overlaps of this candiate prediction with all possible target binding sites on a sequence
			#and choose the one with highest accuracy as the target prediction
			
			#print "check: target site pos", $motifsExp[$seqID], "\n";
			#print "check: predict site pos", $startPos," ",$endPos, " $width\n";
			my @startPosPairs = split(',',$motifsExp[$seqID]); #support multiple target motifs in a sequence
			my $nTargetSites =scalar @startPosPairs;
			my %coveredSites=();	#which sites have been covered?
			
			my $accuracyMax=0; my $sensitivityMax=0; my $specificityMax=0;
			my $accuracy=0; my $sensitivity=0; my $specificity=0;

			#try to match the maximally covered target site
			for(my $i=0;$i<scalar @startPosPairs;$i++){
				my ($startPos1,$width1)=split(' ',$startPosPairs[$i]);#target sites
				my $endPos1=$startPos1+$width1-1;
				$motifWidth=$width1;
				my ($accuracy,$sensitivity,$specificity)=
					PMotif::getAccuracyScore($startPos,$endPos,$startPos1,$endPos1);
				if($accuracy > 0){
					$coveredSites{$startPos1}++;
				}
				if($accuracy>$accuracyMax){
					$accuracyMax = $accuracy;
					$sensitivityMax= $sensitivity;
					$specificityMax= $specificity;
				}
			}
			my $nCoveredSites = scalar values %coveredSites;
			
			$accuracy = $accuracyMax;
			#print "check accuracy: $accuracy\n";
			$sensitivity= $sensitivityMax;
			$specificity= $specificityMax;
			$aveAccuracy+=$accuracy;
			push(@seqAccuracyScores,"$bindingTF\t$seqID\t$accuracy\t$sensitivity\t$specificity\t$score\t$motifWidth\t$nSeq");
		}
		
		$aveAccuracy=$aveAccuracy/(scalar @items);	#average accuracy of all predict-target site pair
													#one possible limitation is that this score doesnot
													#cover uncovered targets. So it's a specificity score
													#used to select top motif group predictions
	# 	pack sequence accuracy information
	#   Export format:  TF aveAccuracy: TF seqID accuracy sensitivity specificity score motifwidth nSeq, 
	# 					TF seqID accuracy sensitivity specificity score motifwidth nSeq,
	# 					TF seqID accuracy sensitivity specificity score motifwidth nSeq,
		my $motifRecord = join(',', @seqAccuracyScores);
		$motifRecord = "$bindingTF $aveAccuracy:$motifRecord"; 
		push(@motifAccuracyScores,$motifRecord);
		@seqAccuracyScores=();
    }    
    
    #report only the TOP $nMotif accuracy scores for all predicted-target site pairs
	#ranked by the average accuracy of predicted sites (so not based on CA,
	#but on site level specificity).
    if($nScore!=0){
		my @topScores=PMotif::getTopScores($nScore,\@motifAccuracyScores);
		#unpack the sequence accuracy information
		my @seqAccuracyScores=();
		foreach my $motifRecord(@topScores){
			my ($head,$motifInfo)=split(':',$motifRecord);
			my @theSeqScores = split(',',$motifInfo);
			push(@seqAccuracyScores,@theSeqScores);
		}
		return @seqAccuracyScores;
    }
    else{
        my @topScores=@motifAccuracyScores;
        my @seqAccuracyScores=();
        foreach my $motifRecord(@topScores){
            my ($head,$motifInfo)=split(':',$motifRecord);
            my @theSeqScores = split(',',$motifInfo);
            push(@seqAccuracyScores,@theSeqScores);
        }
        return @seqAccuracyScores;
    }
}



#------------------------------------------------------------------------
#function: calculate the convergence and accuracy for all motifs in an output file of AlignACE
#input:    a filename of alignACE output
#          $nScore Number of scores to report
#          $nMotif number of motifs to search
#          $voteThreshold percentage of voting sequences to be regarded as consensus motif region
#return:   accuracy values and convergence score  for the discovered motifs along with its statistical scores
#          $aveAccuracy $score $nSeq $motifWidth
sub getConvergeAccuracy{
	#a good way to handle variable number of arguments and default values
	my ($filename,$nMotif,$nScore,$voteThreshold,$algorithm)=@_;
	$nMotif=1 if $nMotif ==0; #number of motifs to be reported
	$nScore=0 if $nScore ==0; #number of top scores to report among the reported motifs,0-all,k-given no.
	$voteThreshold=0.5 if $voteThreshold ==0; #capture the overlap region with over 50% agree the motif is located
	$algorithm="AlignACE" if $algorithm eq "";

    if($nScore>$nMotif && $nMotif!=0){
	print "Error getConvergeAccuracy:: nScore($nScore) should be smaller than nMotif ($nMotif)\n";
	exit;
    }

    #load the experimental motif data
    my $knownSiteFile="./knownsites.txt";
    my @motifsExp =  PMotif::getMotifInfo($filename,$knownSiteFile);
    $filename=~/(.*)\..*/;
    my $bindingTF = $1;

   my @batchFolders=();
   # Open the folder
   unless(opendir(FOLDER, "./")) {
       print "Cannot open current folder !\n";
       exit;
   }
   # Read the folder, ignoring special entries "." and ".."
   my @dataSetFolders = grep (/^step6$algorithm.*/, readdir(FOLDER));

   my @motifRuns=();#motif information of all runs
   
   my $minMotifs=50000;
   my @motifsPredict=();
   for(my $i=0;$i<scalar @dataSetFolders;$i++){
       #extract K  predicted motifs info
       my $pathfilename="";
       if($algorithm eq "AlignACE")
       {@motifsPredict = PMAlignACE::parseMotifInfo("./$dataSetFolders[$i]/$filename",$nMotif);}
       elsif ($algorithm eq "MEME")
       {@motifsPredict = PMMeme::parseMotifInfo("./$dataSetFolders[$i]/$filename",$nMotif);}
       elsif($algorithm eq "MDScan")
       {@motifsPredict = PMMDScan::parseMotifInfo("./$dataSetFolders[$i]/$filename",$nMotif);}
       elsif($algorithm eq "MotifSampler")
       {@motifsPredict = PMMotifSampler::parseMotifInfo("./$dataSetFolders[$i]/$filename",$nMotif);}
       elsif($algorithm eq "BioProspector")
       {@motifsPredict = PMBioProspector::parseMotifInfo("./$dataSetFolders[$i]/$filename",$nMotif);}
       else {print "PMotif::getConvergeAveAccuracy: I don't recognize this algorithm $algorithm\n";}
       
       if( scalar @motifsPredict<$minMotifs){
	   $minMotifs=scalar @motifsPredict;
       }
       push(@motifRuns, \@motifsPredict);
   }

    #print "\n$filename\n";
    #load the experimental motif data
    #my $knownSiteFile="./knownsites.txt";
    #my @motifsExp =  PMotif::getMotifInfo($filename,$knownSiteFile);
 
    #calculate accurary
    my $nSeq= scalar @motifsExp;

    my @motifAccuracyScores=();
    for(my $i=0; $i< $minMotifs; $i++) { #for each discovered motif
	#print "\n*********************motif $i\n";
	my $motifID=$i;
	my $motifAveAccuracyScore=0;
	my $startPosListRuns=[]; #2D array
	my $endPosListRuns=[];
	my $nRuns = scalar @motifRuns;

	#init the motif location matri
	for(my $k1=0;$k1<$nSeq;$k1++){
	    for(my $k2=0;$k2<$nRuns;$k2++){
		$$startPosListRuns[$k1][$k2]=-1;
		$$endPosListRuns[$k1][$k2]=-1;
	    }
	}

	my @scores=();
	for( my $j=0; $j<$nRuns;$j++) {
	    #for all the kth motif in each run
	    #CUSTOMIZE: add more algorithms
	    my @curMotifSet =();
	    if($algorithm eq "AlignACE")
		{@curMotifSet = PMAlignACE::parseMotifInfo("./$dataSetFolders[$j]/$filename",$nMotif);}
	    elsif ($algorithm eq "MEME")
	    	{@curMotifSet = PMMeme::parseMotifInfo("./$dataSetFolders[$j]/$filename",$nMotif);}
	    elsif($algorithm eq "MDScan")
	    	{@curMotifSet = PMMDScan::parseMotifInfo("./$dataSetFolders[$j]/$filename",$nMotif);}
	    elsif($algorithm eq "MotifSampler")
	     	{@curMotifSet = PMMotifSampler::parseMotifInfo("./$dataSetFolders[$j]/$filename",$nMotif);}
	    elsif($algorithm eq "BioProspector")
		{@curMotifSet = PMBioProspector::parseMotifInfo("./$dataSetFolders[$j]/$filename",$nMotif);}
	    else {print "PMotif::getConvergeAveAccuracy: I don't recognize this algorithm $algorithm\n";}
	    	    
	    my $record=$curMotifSet[$i];#the iTH motif
	    my ($itemList,$score)=split(':',$record);
	    push(@scores,$score);
	    my @items =  split(',',$itemList);
	    
	    my @accuracyList=();
	    foreach my $item (@items){
		my ($seqID, $startPos, $width)=split(' ',$item);
		my $endPos=$startPos+$width-1;
		$$startPosListRuns[$seqID-1][$j]= $startPos;
		$$endPosListRuns[$seqID-1][$j] = $endPos;
		#print "$seqID $j : $startPos $endPos $width***\n";
	    }
	}
	my $aveMotifScore=0;
	foreach my $theScore(@scores){
	    $aveMotifScore+=$theScore;
	}
	$aveMotifScore=$aveMotifScore/scalar @scores;

	#compute the convergence score
	my @seqAccuracyScores=();#store the average accuracy of all motifs discovered by the algorithm
	my $aveAccuracy=0;
	for(my $ii=0; $ii<$nSeq; $ii++){
	    my @leftBound=();
	    my @rightBound=();
	    for(my $jj=0; $jj<$nRuns; $jj++){
		push(@leftBound, $$startPosListRuns[$ii][$jj]);
		push(@rightBound, $$endPosListRuns[$ii][$jj]);
	    }	    
	    my($overlapStartPos,$overlapEndPos,$spanWidth)=
			PMotif::getOverlapInfo(\@leftBound, \@rightBound,$voteThreshold);
	    
	    if($overlapStartPos>=0 && $overlapEndPos>=0 && $spanWidth>0){
		my ($startPos1,$width1)=split(' ',$motifsExp[$ii]);#get experiment motif sites
		    my $endPos1=$startPos1+$width1-1;
		my $motifWidth=$width1; #assume all training sequences have same motif length.
	    
		#calculate accuracy and convergence score for a training sequence
		my ($accuracy,$sensitivity,$specificity)=
			PMotif::getAccuracyScore($overlapStartPos,$overlapEndPos,$startPos1,$endPos1);	   
		$aveAccuracy +=$accuracy; 
		my $convergenceScore= ($overlapEndPos-$overlapStartPos+1)/$spanWidth;
		#print "\n== $overlapStartPos $overlapEndPos $spanWidth\n";
		#show the convergence/accuracy of all training sequences of each motif
		push(@seqAccuracyScores,"$bindingTF\t$ii\t$accuracy\t$sensitivity\t$specificity\t$convergenceScore\t$motifWidth\t$nSeq");
	    }
	}
	$aveAccuracy =$aveAccuracy/$nSeq;
	#pack sequence accuracy information
        my $motifRecord = join(',', @seqAccuracyScores);
	$motifRecord = "$bindingTF $aveAccuracy:$motifRecord";
	push(@motifAccuracyScores,$motifRecord);
    }
    

    #report only the TOP $nMotif accuracy score
    if($nScore!=0){
	my @topScores=PMotif::getTopScores($nScore,\@motifAccuracyScores);
	#unpack the sequence accuracy information
	my @seqAccuracyScores=();
	foreach my $motifRecord(@topScores){
	    my ($head,$motifInfo)=split(':',$motifRecord);
	    my @theSeqScores = split(',',$motifInfo);
	    push(@seqAccuracyScores,@theSeqScores);
	}
	return @seqAccuracyScores;
    }
    else{
        my @topScores=@motifAccuracyScores;
        my @seqAccuracyScores=();
        foreach my $motifRecord(@topScores){
            my ($head,$motifInfo)=split(':',$motifRecord);
            my @theSeqScores = split(',',$motifInfo);
            push(@seqAccuracyScores,@theSeqScores);
        }
        return @seqAccuracyScores;
    }
}





#------------------------------------------------------------------------
#function: calculate the convergence and accuracy for all motifs in an output file of AlignACE
#input:    a filename of alignACE output
#          $nScore Number of scores to report
#          $nMotif number of motifs to search
#          $voteThreshold percentage of voting sequences to be regarded as consensus motif region
#          algorithm name
#return:   accuracy values and convergence score  for the discovered motifs along with its statistical scores
#          $aveAccuracy $score $nSeq $motifWidth
#$bindingTF\t$aveAccuracy\t          $aveConvergenceScore\t  $aveMotifWidth\t        $nSeq
#Ada         0.238095238095238       0.423423423423423       18.6666666666667        3
#AraC        0.215017825311943       0.583333333333333       9.91666666666667        12

sub getConvergeAveAccuracy{
	#a good way to handle variable number of arguments and default values
	my ($filename,$nMotif,$nScore,$voteThreshold,$algorithm)=@_;
	$nMotif=1 if $nMotif ==0; #number of motifs to be reported
	$nScore=0 if $nScore ==0; #number of top scores to report among the reported motifs,0-all,k-given no.
	$voteThreshold=0.5 if $voteThreshold ==0; #capture the overlap region with over 50% agree the motif is located
	$algorithm="AlignACE" if $algorithm eq "";
    
    if($nScore>$nMotif && $nMotif!=0){
	print "Error getAccuracy:: nScore($nScore) should be smaller than nMotif ($nMotif)\n";
	exit;
    }
    
	#load the experimental motif data
    my $knownSiteFile="./knownsites.txt";
    my @motifsExp =  PMotif::getMotifInfo($filename,$knownSiteFile);
    $filename=~/(.*)\..*/;
    my $bindingTF = $1;

   my @batchFolders=();
   # Open the folder
   unless(opendir(FOLDER, "./")) {
       print "Cannot open current folder !\n";
       exit;
   }
   # Read the folder, ignoring special entries "." and ".."
   my @dataSetFolders = grep (/^step6$algorithm.*/, readdir(FOLDER));

   my @motifRuns=();#motif information of all runs
   
   my $minMotifs=50000;
    my @motifsPredict=();
   for(my $i=0;$i<scalar @dataSetFolders;$i++){
   	    #extract K  predicted motifs info
   	    my $pathfilename="";
	    #CUSTOMIZE: add more algorithms	 
	    my @motifsPredict=();   
	    if($algorithm eq "AlignACE")
	    	{@motifsPredict = PMAlignACE::parseMotifInfo("./$dataSetFolders[$i]/$filename",$nMotif);}
	    elsif ($algorithm eq "MEME")
	    	{@motifsPredict = PMMeme::parseMotifInfo("./$dataSetFolders[$i]/$filename",$nMotif);}
	    elsif($algorithm eq "MDScan")
	    	{@motifsPredict = PMMDScan::parseMotifInfo("./$dataSetFolders[$i]/$filename",$nMotif);}
	    elsif($algorithm eq "MotifSampler")
	     	{@motifsPredict = PMMotifSampler::parseMotifInfo("./$dataSetFolders[$i]/$filename",$nMotif);}
	    elsif($algorithm eq "BioProspector")
	    	{@motifsPredict = PMBioProspector::parseMotifInfo("./$dataSetFolders[$i]/$filename",$nMotif);}
	    else {print "PMotif::getConvergeAveAccuracy: I don't recognize this algorithm $algorithm\n";}
	    
	    if( scalar @motifsPredict<$minMotifs){
	    	$minMotifs=scalar @motifsPredict;
	    }
	    push(@motifRuns, \@motifsPredict);
   }

    #print "\n$filename\n";
    #load the experimental motif data
    #my $knownSiteFile="./knownsites.txt";
    #my @motifsExp =  PMotif::getMotifInfo($filename,$knownSiteFile);
 
    #calculate accurary
    my $nSeq= scalar @motifsExp;

    my @motifAccuracyScores=();
    for(my $i=0; $i< $minMotifs; $i++) { #for each discovered motif
	my $motifID=$i;
	my $motifAveAccuracyScore=0;
	my $startPosListRuns=[]; #2D array
	my $endPosListRuns=[];
	my $nRuns = scalar @motifRuns;

	#init the motif location matri
	for(my $k1=0;$k1<$nSeq;$k1++){
	    for(my $k2=0;$k2<$nRuns;$k2++){
		$$startPosListRuns[$k1][$k2]=-1;
		$$endPosListRuns[$k1][$k2]=-1;
	    }
	}

	my @scores=();
	for( my $j=0; $j<$nRuns;$j++) {
	    #for all the kth motif in each run
	    my @curMotifSet = ();
	    #CUSTOMIZE: add more algorithms
	    if($algorithm eq "AlignACE")
		{@curMotifSet = PMAlignACE::parseMotifInfo("./$dataSetFolders[$j]/$filename",$nMotif);}
	    elsif ($algorithm eq "MEME")
	    	{@curMotifSet = PMMeme::parseMotifInfo("./$dataSetFolders[$j]/$filename",$nMotif);}
	    elsif($algorithm eq "MDScan")
	    	{@curMotifSet = PMMDScan::parseMotifInfo("./$dataSetFolders[$j]/$filename",$nMotif);}
	    elsif($algorithm eq "MotifSampler")
	     	{@curMotifSet = PMMotifSampler::parseMotifInfo("./$dataSetFolders[$j]/$filename",$nMotif);}
	    elsif($algorithm eq "BioProspector")
		{@curMotifSet = PMBioProspector::parseMotifInfo("./$dataSetFolders[$j]/$filename",$nMotif);}
	    else {print "PMotif::getConvergeAveAccuracy: I don't recognize this algorithm $algorithm\n";}
	    	    
	    my $record=$curMotifSet[$i];#the iTH motif

	    my ($itemList,$score)=split(':',$record);
	    push(@scores,$score);
	    my @items =  split(',',$itemList);
	    
	    my @accuracyList=();
	    foreach my $item (@items){
		my ($seqID, $startPos, $width)=split(' ',$item);
		my $endPos=$startPos+$width-1;
		$$startPosListRuns[$seqID-1][$j]= $startPos;
		$$endPosListRuns[$seqID-1][$j] = $endPos;
		#print "$seqID $j : $startPos $endPos $width***\n";
	    }
	}
	my $aveMotifScore=0;
	foreach my $theScore(@scores){
	    $aveMotifScore+=$theScore;
	}
	$aveMotifScore=$aveMotifScore/scalar @scores;

	#compute the convergence score
	my $aveAccuracy=0;
        my $aveSensitivity;
        my $aveSpecificity;
 
	my $aveConvergenceScore=0;
	my $aveMotifWidth=0;
	for(my $ii=0; $ii<$nSeq; $ii++){
	    my @leftBound=();
	    my @rightBound=();
	    for(my $jj=0; $jj<$nRuns; $jj++){
		push(@leftBound, $$startPosListRuns[$ii][$jj]);
		push(@rightBound, $$endPosListRuns[$ii][$jj]);
	    }	

	    my($overlapStartPos,$overlapEndPos,$spanWidth)=
			PMotif::getOverlapInfo(\@leftBound, \@rightBound,$voteThreshold);
	    
	    if($overlapStartPos>=0 && $overlapEndPos>=0 && $spanWidth>0){
	        #print "overlap: $overlapStartPos--$overlapEndPos: $spanWidth\n";
		my ($startPos1,$width1)=split(' ',$motifsExp[$ii]);#get experiment motif sites
		    my $endPos1=$startPos1+$width1-1;
		my $motifWidth=$width1; #assume all training sequences have same motif length.

		#calculate accuracy and convergence score for a training sequence
		my ($accuracy,$sensitivity,$specificity)= PMotif::getAccuracyScore($overlapStartPos,$overlapEndPos,$startPos1,$endPos1);    
		my $convergenceScore= ($overlapEndPos-$overlapStartPos+1)/$spanWidth;
		#print "\n== $overlapStartPos $overlapEndPos $spanWidth\n";
		#show the convergence/accuracy of all training sequences of each motif
		$aveAccuracy+= $accuracy;
		$aveSensitivity+=$sensitivity;
		$aveSpecificity+=$specificity;
		$aveConvergenceScore+= $convergenceScore;
		$aveMotifWidth+=$motifWidth;
	    }
	}
	$aveAccuracy =$aveAccuracy/$nSeq;
	$aveSensitivity=$aveSensitivity/$nSeq;
	$aveSpecificity=$aveSpecificity/$nSeq;
	$aveConvergenceScore=$aveConvergenceScore/$nSeq;
	$aveMotifWidth=$aveMotifWidth/$nSeq;

	#pack sequence accuracy information
    my $motifRecord = "$bindingTF\t$aveAccuracy\t$aveSensitivity\t$aveSpecificity\t$aveConvergenceScore\t$aveMotifWidth\t$nSeq";
	$motifRecord = "$bindingTF $aveAccuracy:$motifRecord";
	push(@motifAccuracyScores,$motifRecord);
    }
    
    #report only the TOP $nMotif accuracy score
    if($nScore!=0){
	my @topScores=PMotif::getTopScores($nScore,\@motifAccuracyScores);
	#unpack the sequence accuracy information
	my @seqAccuracyScores=();
	foreach my $motifRecord(@topScores){
	    my ($head,$motifInfo)=split(':',$motifRecord);
	    my @theSeqScores = split(',',$motifInfo);
	    push(@seqAccuracyScores,@theSeqScores);
	}
	return @seqAccuracyScores;
    }
    else{
        my @topScores=@motifAccuracyScores;
        my @seqAccuracyScores=();
        foreach my $motifRecord(@topScores){
            my ($head,$motifInfo)=split(':',$motifRecord);
            my @theSeqScores = split(',',$motifInfo);
            push(@seqAccuracyScores,@theSeqScores);
        }
        return @seqAccuracyScores;
    }
}







1;

__END__

=head1 NAME

PMotif subroutines
#Statistics::Descriptive::Sparse
#----------------------------------
#add_data  with empty array will delete all existing data
#count
#mean
#sum
#variance
#pseudo_variance
#min
#max
#mindex
#maxdex
#standard_deviation
#sample_range

#Statistics::Descriptive::Full
#----------------------------------
#All methods above and:
#get_data
#sort_data
#presorted
#percentile
#median
#trimmed_mean
#harmonic_mean
#mode
#geometric_mean
#frequency_distribution
#least_squares_fit

