#!/usr/bin/perl
# PMDEnsembleLite Package
# Function: finding the motifs using ensemble algorithm PMDEnsemble
# Author: Jianjun Hu (hujianju@purdue.edu)
# Kihara Laboratory of Bioinformatics
# Department of Biological Sciences
# Purdue University, West Lafayette, IN, 47906.
#
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
# Last modified: 10-25-2004v
#export PERL5LIB=~/motif/bin:/usr/local/kihara/lib/perl/5.8.4:$PERL5LIB .bashrc for cluster

use strict;
use PMotif;
use PMMDScan;	#deterministic
use PMMeme;		#deterministic
use PMMotifSampler;	#stochastic
use PMAlignACE;		#stochastic
use PMBioProspector;#stochastic
use PToolbox;
use POSIX qw(ceil floor);

my $debug =0;
my $runType = "parallel";

#-----------------------------------------------------------------------
#	commandline option processing
use Getopt::Std;
my %opts = ();
getopts('hf:c:n:', \%opts);
$0=~/.*\/(.*)/;
# ':'after symbol means it takes arguments, otherwise just a flag which is 1 when set
sub Usage {
print <<"EOF";

Usage: $1 -f<input sequence filename> -c<configuration file> -n <No. Motifs>
Function: find the motifs using ensemble algorithm
Examples:
  	$1 -f Ada.txt -c emd.cfg -m 5
Description:
	-f input sequence filename [step5regulon/Ada.txt]
	-c PMDEnsemble configuration file [emd.cfg]
	-n numer of motifs to find	[5]
EOF
    exit;
}
&Usage() if defined $opts{'h'};
my $filename= $opts{'f'}||"step5regulon/Ada.txt";
my $nMotif=$opts{'n'}||5;
my $configFile=$opts{'c'}||"emd.cfg";

#parallel: parallel version of ensemble folder structure
#		   step6Algorithm/exp-1/step6Algorithm
#local:    local version of ensemble folder structure
#		   step6Algorithm1/, step6Algorithm2...

#------------------------------------------------------------------------
#function: extract all the discovered motifs from output of 
#          PMDEnsembleLite: each motif info seperated by ';'
#          motif position for each sequence separated by ','
#          startPosition and width and score separated by ' '
#		   Key ideas here is to extract motifs from multiple algorithms 
#		   according to the ensemble algorithm configuration file
#input:    filename of PMDEnsembleLite output file, number of motifs to report
#comment:  for PMDEnsembleLite results, we skip the motifs found in reverse sequence
#return:   an array of motif information.
#          seqID startPos width, ...: score    for each motif discovered

#parse the result of local runs. directory structure is like
#step6Algorithm1, step6Algorithm2...
	
#-----------------------------------------------------------------------
#read configuration file and run the standalone programs
unless(open(FILE,$configFile)){
	print "Error in PMDEnsembleListe: cannot find config file $configFile\n";
	exit;
}
my @configData=<FILE>;
@configData=grep !/#.*\n$/,@configData;
my $dataline= join("",@configData);
#print $dataline,"\n\n";
my @groupData = split("Alg:",$dataline);
my $nGroup=scalar @groupData;

#section 1:
chop $groupData[0];
#print $groupData[0];
my @items = split(":",$groupData[0]);
my $nAlgorithm = $items[1];
# print "nAlgorithm = $nAlgorithm\n";

#section 2:
my %motifRank=(); #each element is a rank and an array containing
				  #the predictions of that rank for all algorithms
for(my $i=0;$i<$nMotif;$i++){
	$motifRank{$i}=();
}
for(my $k=1;$k<=$nAlgorithm;$k++){
	my @motifsPredict=(); #predicts for one algorithm
	chop $groupData[$k];
	my @lines = split("\n",$groupData[$k]);
	my ($algorithm, $nRun, $nConfig) = split(":",$lines[0]);
	if($nRun ==0) {
		next;
	}
	#print  "$algorithm, $nRun, $nConfig\n";
	
	for (my $i=0;$i<$nRun;$i++){
		my $dstFolder="step6$algorithm$i";
		if($runType eq "parallel"){
			$dstFolder="step6$algorithm/exp-$i/step6$algorithm";
		}

		if(!(-e $dstFolder)){
			print "Error---PMDEnsembleLite::parseMotifInfo:\n dstFolder($dstFolder) does not exist\n";
			exit;
		}
		#print $filename,"  $k src file  $algorithm\n";
		$filename=~/(.*)\/(.*)$/;
		my $ofile = $2;
		$ofile="$dstFolder/$ofile";
		#print $ofile," target file  $i\n\n";
		
		my @myPredicts=();
		if($algorithm eq "AlignACE")
		{@myPredicts = PMAlignACE::parseMotifInfo("$ofile",$nMotif);}
		elsif ($algorithm eq "MEME")
		{@myPredicts = PMMeme::parseMotifInfo("$ofile",$nMotif);}
		elsif($algorithm eq "MDScan")
		{@myPredicts = PMMDScan::parseMotifInfo("$ofile",$nMotif);}
		elsif($algorithm eq "MotifSampler")
		{@myPredicts = PMMotifSampler::parseMotifInfo("$ofile",$nMotif);}
		elsif($algorithm eq "BioProspector")
		{@myPredicts = PMBioProspector::parseMotifInfo("$ofile",$nMotif);}
		else 
		{
			print "PMDEnsembleLite::parseMotifInfo: I don't recognize this algorithm- $algorithm\n";
			exit:
		}
		for(my $j=0;$j<scalar @myPredicts;$j++){
			$myPredicts[$j]=join(':',$myPredicts[$j],$algorithm);
		}
		my $exportRecord = join(';',@myPredicts);
		#print "$exportRecord ***\n\n";
		push(@motifsPredict, $exportRecord);
	}#end of run
	my $allPredicts = join(';',@motifsPredict);
	@motifsPredict = split(';',$allPredicts);
	my $nPredicts = scalar @motifsPredict;
	my $nPredictExtra = $nPredicts%$nMotif; #these # of top ranks will get one additional predict.
	my $nPredictofGroup = ($nPredicts-$nPredictExtra)/$nMotif;
	#sort according to the score
	@motifsPredict = sort {PToolbox::getMotifScore($b) <=> PToolbox::getMotifScore($a) } @motifsPredict;
	my $startPredictPos=0;
	for(my $iRank =0; $iRank<$nMotif;$iRank++){
		my $myPredictNo = $nPredictofGroup;
		if($iRank < $nPredictExtra){
			$myPredictNo++;
		}
		push(@{$motifRank{$iRank}}, @motifsPredict[$startPredictPos..$startPredictPos+$myPredictNo-1]);
		$startPredictPos += $myPredictNo;
	}
	#print $nPredicts, "  ", $nMotif, "  ", $nPredictofGroup,"  ", "===\n";
	$debug=0;
	print "@motifsPredict 888\n\n" if($debug==1);
}#for algorithm
	
#we report nMotif predictions
my @motifsPredict=(); #output motif predictions
for(my $iPredict=0; $iPredict< $nMotif;$iPredict++){
	#print @{$motifRank{$iPredict}}, "****** $iPredict\n";
	
	#Lite ensemble algorithm: how to combine the results and report the combined
	#predictions
    #Begin:Ensemble algorithms
    #step 1: reorganize the prediction records
    my @ensemblePredicts= @{$motifRank{$iPredict}};
    my @sitePredictList=();
    my %nBindingSitesPred=();#record the number of predicted sites for each output group to calcualte median
    my %nBindingSitesSeq=(); #count the number of predicted binding sites 
						     #used to estimate the number of clusters when do ensemble
    my $nPredictGroup=0;
    foreach my $predictAlgo (@ensemblePredicts){
		#for predictions of one run of an algorithm 
		my @predictAlgoList = split(';',$predictAlgo);	#for mulitple predicts of one run
		$nPredictGroup += scalar @predictAlgoList;
		foreach my $predictGroup (@predictAlgoList){
			my ($record, $score,$algorithm) = split(':',$predictGroup);
			my @items = split(',',$record);
			my %myBindingSitesNo=();	#count the number of predicted sites on each sequence for one group of prediction
			foreach my $item (@items){
				my @groupItem = split(' ',$item);
				$nBindingSitesSeq{$groupItem[0]}++;
				$myBindingSitesNo{$groupItem[0]}++;
				push(@sitePredictList, "$item $score $algorithm");
			}
			foreach (keys %myBindingSitesNo){
				$nBindingSitesPred{$_}= "$nBindingSitesPred{$_} $myBindingSitesNo{$_}";
			}
		}
    }
	
    foreach my $key( sort  {$a <=> $b} keys %nBindingSitesSeq){
	    my $list= $nBindingSitesPred{$key};
	    $list=PToolbox::cleanString($list);
	    my $nSitesEstimate=1;
	    my @items= split(' ',$list);

	    PToolbox::printH(@items) if ($debug ==1);

	    my $stat = Statistics::Descriptive::Full->new();
	    $stat->add_data(@items);
	    #$nSitesEstimate = $stat->median();
		$nSitesEstimate = int($stat->mean()+0.5);
		#$nSitesEstimate = int($stat->max());
		#$nSitesEstimate = int(($stat->min+$stat->max()+0.5)/2);
	    #$nSitesEstimate = $nBindingSitesSeq{$key}/$nPredictGroup;
	    #$nSitesEstimate = ceil($nSitesEstimate);
	    #$nSitesEstimate = int($nSitesEstimate+0.5);
	    
	    $nBindingSitesSeq{$key}=$nSitesEstimate;
	    #print $key,"  ", $nBindingSitesSeq{$key}," ***\n\n";
    }

	#load the experimental motif data
	my $nSequences = `grep ">" $filename | wc -l`;
	chomp $nSequences;
	#print "nSequences = $nSequences *$filename*\n";
	
	#step 2: for each sequence we do ensemble algorithm
	my $outputRecord="";
	for(my $i=0;$i<$nSequences;$i++){
		my $outputRecordSeq="";
		my $nSitesEstimate= $nBindingSitesSeq{$i};
		
		my @startPos = ();
		my @widthList =();
		my @endPos=();
		my @scores = ();
		my @algorithms=();
		foreach my $seqPredict (@sitePredictList){
		#for predictions of each sequence
			my @items = split(' ', $seqPredict);
			if($items[0] == $i){
				push(@startPos,$items[1]);
				push(@widthList,$items[2]);
				push(@endPos, $items[1]+$items[2]-1);
				push(@scores, $items[3]);
				push(@algorithms, $items[4]);
				#print $itsems[1],"\n";
			}
		}
	
		#Ensemble algorithm: based on histogram of votes
		#combine all the results using histomgram method
		#print "********* Histogram method for ensemble algorithm\n";
		#print "nSequence === $nSequences  \n";
		#$nSitesEstimate
		#$debug=0;
		PToolbox::printH(@startPos)if($debug==1);
		#$debug=0;
		print "nSitesEstimate = $nSitesEstimate 777\n" if($debug==1);
		#print $nSitesEstimate, "\t";
		
		my @predictList =
			PMotif::mergeHistogram(\@startPos, \@endPos, $nSequences,0, $nSitesEstimate);

		
		foreach(@predictList){
			if($outputRecordSeq eq ""){
				$outputRecordSeq="$i $_";
			}
			else{
				$outputRecordSeq=join(',', $outputRecordSeq, "$i $_");
			}
		}
		if($outputRecord eq ""){
			$outputRecord = $outputRecordSeq;
		}
		else{
			$outputRecord = join(',',$outputRecord,$outputRecordSeq);
		}
	}
	$outputRecord ="$outputRecord:1.0";
	push(@motifsPredict,$outputRecord);
	print"==\n@motifsPredict ==\n\n" if($debug==1);
	
	#Extract the predicted motif sequenences and write to
	#directory step6PMDEnsembleLite
	#To do for users
	
	#print $outputRecord, "*****\n";
	my ($predictList,$score)= split(':',$outputRecord);
	my @sitePredicts = split(',',$predictList);
	unless(open(INPUT_SEQ,"$filename")){
		print "Error in PMDEnsembleListe: cannot find file $filename\n";
		exit;
	}
	my @inputSequences=<INPUT_SEQ>;
	@inputSequences = grep !/>.*/, @inputSequences;

	print "\nMotif $iPredict:\n";	
	for(my $iSite=0;$iSite< scalar @sitePredicts;$iSite++){
		my ($seqID, $pos, $width)= split(' ', $sitePredicts[$iSite]);
		#print "$seqID, $pos, $width\n";
		my $siteSeq = substr($inputSequences[$seqID],$pos,$width);
		print $siteSeq, " $seqID $pos $width\n";
	}
	
}#$iPredict
